<?php

/**
 * Outputs the HTML for a form field based on the provided field data.
 *
 * This function dynamically generates the HTML for different types of form fields, including text, password, email, number,
 * textarea, select, radio, checkbox, and file. It also sanitizes the field data using wp_kses() and applies filters to
 * customize the allowed HTML tags.
 *
 * @param array $field An associative array containing the field data. The array should have the following keys:
 *                     - 'type': The type of the field (e.g., 'text', 'textarea', 'select', etc.).
 *                     - 'name': The name attribute of the field.
 *                     - 'value': The value attribute of the field.
 *                     - 'options': (optional) An associative array of options for select, radio, and checkbox fields.
 *
 * @return void
 */
function ld_dashboard_get_field_html( $field ) {
	// Initialize allowed HTML array
	$allowed_html = array();

	// Determine the type of field and set allowed attributes dynamically
	if ( isset( $field['type'] ) && ! empty( $field['type'] ) ) {
		$allowed_html['input'] = array(
			'type'    => true,
			'name'    => true,
			'value'   => true,
			'class'   => true,
			'id'      => true,
			'checked' => true,
		);

		// Loop through the field attributes and add them to the allowed HTML array
		foreach ( $field as $key => $value ) {
			if ( ! isset( $allowed_html['input'][ $key ] ) ) {
				$allowed_html['input'][ $key ] = true;
			}
		}

		// Additional allowed HTML elements based on the field type
		if ( 'textarea' === $field['type'] ) {
			$allowed_html['textarea'] = array(
				'name'  => true,
				'class' => true,
				'id'    => true,
				'rows'  => true,
				'cols'  => true,
			);
			foreach ( $field as $key => $value ) {
				if ( ! isset( $allowed_html['textarea'][ $key ] ) ) {
					$allowed_html['textarea'][ $key ] = true;
				}
			}
		} elseif ( 'select' === $field['type'] ) {
			$allowed_html['select'] = array(
				'name'  => true,
				'class' => true,
				'id'    => true,
			);
			$allowed_html['option'] = array(
				'value'    => true,
				'selected' => true,
			);
		}

		$allowed_html['span'] = array(
			'class' => true,
		);
	}

	// Echo the sanitized HTML output
	echo wp_kses( ld_dashboard_get_input_field_html( $field ), apply_filters( 'ld_dashboard_fields_allowed_html', $allowed_html ) );
}



/**
 * Generates the HTML for a form field based on the provided field data.
 *
 * This function dynamically generates the HTML for different types of form fields, including text, password, email, number,
 * textarea, select, radio, checkbox, and file. It also sanitizes the field data using esc_attr() and esc_textarea() functions
 * and applies filters to customize the allowed HTML tags.
 *
 * @param array $field An associative array containing the field data. The array should have the following keys:
 *                     - 'type': The type of the field (e.g., 'text', 'textarea', 'select', etc.).
 *                     - 'name': The name attribute of the field.
 *                     - 'value': The value attribute of the field.
 *                     - 'options': (optional) An associative array of options for select, radio, and checkbox fields.
 *
 * @return string The generated HTML for the form field.
 */
function ld_dashboard_get_input_field_html( $field ) {

	$html = '';
	if( is_array( $field ) && isset( $field['type'] )){
		switch ( $field['type'] ) {

			case 'text':
			case 'password':
			case 'email':
			case 'number':
			case 'url':
			case 'tel':
			case 'hidden':
			case 'date':
			case 'time':
			case 'datetime-local':
				$html .= '<input type="' . esc_attr( $field['type'] ) . '" name="' . esc_attr( $field['name'] ) . '" value="' . esc_attr( $field['value'] ) . '">';
				break;
	
			case 'checkbox':
				$checked = ! empty( $field['value'] ) ? 'checked' : '';
				$html   .= '<input type="checkbox" name="' . esc_attr( $field['name'] ) . '" value="yes" ' . $checked . '>';
				break;
	
			case 'radio':
				if ( ! empty( $field['options'] ) && is_array( $field['options'] ) ) {
					foreach ( $field['options'] as $option_value => $option_label ) {
						$checked = ( $field['value'] == $option_value ) ? 'checked' : '';
						$html   .= '<label><input type="radio" name="' . esc_attr( $field['name'] ) . '" value="' . esc_attr( $option_value ) . '" ' . $checked . '> ' . esc_html( $option_label ) . '</label>';
					}
				}
				break;
	
			case 'textarea':
				$html .= '<textarea name="' . esc_attr( $field['name'] ) . '">' . esc_textarea( $field['value'] ) . '</textarea>';
				break;
	
			case 'select':
				if ( ! empty( $field['options'] ) && is_array( $field['options'] ) ) {
					$html .= '<select name="' . esc_attr( $field['name'] ) . '">';
					foreach ( $field['options'] as $option_value => $option_label ) {
						$selected = ( $field['value'] == $option_value ) ? 'selected' : '';
						$html    .= '<option value="' . esc_attr( $option_value ) . '" ' . $selected . '>' . esc_html( $option_label ) . '</option>';
					}
					$html .= '</select>';
				}
				break;
	
			case 'file':
				$html .= '<input type="file" name="' . esc_attr( $field['name'] ) . '">';
				break;
	
			default:
				$html .= '<input type="text" name="' . esc_attr( $field['name'] ) . '" value="' . esc_attr( $field['value'] ) . '">';
				break;
		}
	}

	// If the field type is 'password', add the visibility toggle
	if ( is_array( $field ) && isset( $field['type'] ) && ( 'password' === $field['type'] ) ) {
		$html .= '<span class="ld-dashboard-password-toggle dashicons dashicons-visibility"></span>';
	}

	return apply_filters( 'ld_dashboard_fields_html', $html );
}

add_action( 'init', 'ld_dashboard_submit_user_zoom_credentials' );
function ld_dashboard_submit_user_zoom_credentials() {

	if ( isset( $_POST['action'] ) && 'update-zoom-settings' === $_POST['action'] ) {
		if ( isset( $_POST['_wpnonce'] ) && wp_verify_nonce( sanitize_text_field( wp_unslash( $_POST['_wpnonce'] ) ), 'update-zoom-settings' ) ) {

			if ( isset( $_POST['zoom_api_key'] ) && '' != $_POST['zoom_api_key'] ) {
				update_user_meta( get_current_user_id(), 'zoom_api_key', sanitize_text_field( wp_unslash( $_POST['zoom_api_key'] ) ) );
			} else {
				update_user_meta( get_current_user_id(), 'zoom_api_key', '' );
			}
			if ( isset( $_POST['zoom_api_secret'] ) && '' != $_POST['zoom_api_secret'] ) {
				update_user_meta( get_current_user_id(), 'zoom_api_secret', sanitize_text_field( wp_unslash( $_POST['zoom_api_secret'] ) ) );
			} else {
				update_user_meta( get_current_user_id(), 'zoom_api_secret', '' );
			}
			if ( isset( $_POST['zoom_account_id'] ) && '' != $_POST['zoom_account_id'] ) {
				update_user_meta( get_current_user_id(), 'zoom_account_id', sanitize_text_field( wp_unslash( $_POST['zoom_account_id'] ) ) );
			} else {
				update_user_meta( get_current_user_id(), 'zoom_account_id', '' );
			}
			if ( isset( $_POST['zoom_email'] ) && '' != $_POST['zoom_email'] ) {
				update_user_meta( get_current_user_id(), 'zoom_email', sanitize_text_field( wp_unslash( $_POST['zoom_email'] ) ) );
			} else {
				update_user_meta( get_current_user_id(), 'zoom_email', '' );
			}

			if ( isset( $_POST['enable_browser_join'] ) && '' != $_POST['enable_browser_join'] ) {
				update_user_meta( get_current_user_id(), 'enable_browser_join', sanitize_text_field( wp_unslash( $_POST['enable_browser_join'] ) ) );
			} else {
				update_user_meta( get_current_user_id(), 'enable_browser_join', '' );
			}

			if ( isset( $_POST['client_id'] ) && '' != $_POST['client_id'] ) {
				update_user_meta( get_current_user_id(), 'client_id', sanitize_text_field( wp_unslash( $_POST['client_id'] ) ) );
			} else {
				update_user_meta( get_current_user_id(), 'client_id', '' );
			}

			if ( isset( $_POST['client_secret'] ) && '' != $_POST['client_secret'] ) {
				update_user_meta( get_current_user_id(), 'client_secret', sanitize_text_field( wp_unslash( $_POST['client_secret'] ) ) );
			} else {
				update_user_meta( get_current_user_id(), 'client_secret', '' );
			}
			do_action( 'ld_dashboard_save_zoom_fields' );

		}
	}
}



