<?php
/**
 * Shortcode to display instructor information and their courses.
 *
 * @param array $atts Shortcode attributes.
 * @return string Rendered HTML content.
 */
function ld_dashboard_instructors_callback( $atts ) {
    global $post;
    ob_start();

    // Set default number of columns for display.
    $cols = isset( $atts['col'] ) ? $atts['col'] : 4;

    // Allow developers to modify the number of columns via a filter.
    $cols = apply_filters( 'ld_dashboard_instructors_columns', $cols );

    // Safely fetch the instructor username from query vars.
    $instructor_username = ( isset( $GLOBALS['wp_query'] ) && function_exists( 'get_query_var' ) )
        ? get_query_var( 'instructor_username', '' )
        : '';

    // Fetch instructor details if username is provided.
    $instructor = ! empty( $instructor_username ) ? get_user_by( 'login', $instructor_username ) : null;

    // Fetch instance of Ld_Dashboard_Functions.
    $function_obj = class_exists( 'Ld_Dashboard_Functions' ) ? Ld_Dashboard_Functions::instance() : null;

    // Fetch the instructors page URL.
    $instructors_page_url = $function_obj ? $function_obj->ld_dashboard_get_url( 'instructors' ) : '';

    // Get LearnDash dashboard settings.
    $ld_dashboard_settings_data = $function_obj ? $function_obj->ld_dashboard_settings_data() : array();
    $welcome_screen = isset( $ld_dashboard_settings_data['welcome_screen'] ) ? $ld_dashboard_settings_data['welcome_screen'] : array();

    // Set dashboard landing cover if an image is provided.
    $dashboard_landing_cover = isset( $welcome_screen['welcomebar_image'] ) && ! empty( $welcome_screen['welcomebar_image'] )
        ? "background-image: url({$welcome_screen['welcomebar_image']});"
        : '';

    if ( ! is_object( $instructor ) ) {
        ?>
        <div class="ld-dashboard-instructors-shortcode-wrapper">
            <div class="ld-dashboard-instructors-list" style="grid-template-columns: repeat(<?php echo esc_attr( $cols ); ?>, 1fr);">
                <?php
                $instructors = get_users( 'orderby=nicename&role=ld_instructor' );

                // Allow developers to filter the instructor list.
                $instructors = apply_filters( 'ld_dashboard_instructors_list', $instructors );

                if ( is_array( $instructors ) && ! empty( $instructors ) ) {
                    foreach ( $instructors as $instructor ) {
                        // Allow developers to filter individual instructor data.
                        $instructor = apply_filters( 'ld_dashboard_instructor_data', $instructor );

                        $instructor_courses = Ld_Dashboard_Public::get_instructor_courses_list( $instructor->ID );

                        // Allow developers to filter the instructor courses.
                        $instructor_courses = apply_filters( 'ld_dashboard_instructor_courses', $instructor_courses, $instructor );
                        ?>
                        <div class="ld-dashboard-instructor-list-grid">
                            <div class="ld-dashboard-instructors-list-cover-bg" style="<?php echo esc_attr( $dashboard_landing_cover ); ?>"></div>
                            <a href="<?php echo esc_url( $instructors_page_url ) . esc_attr( $instructor->user_login ); ?>" class="ld-dashboard-instructor-list">
                                <div class="ld-dashboard-instructor-profile-photo">
                                    <?php
                                    // Allow developers to filter the instructor avatar.
                                    echo apply_filters( 'ld_dashboard_instructor_avatar', wp_kses_post( get_avatar( $instructor->ID, 320 ) ), $instructor );
                                    ?>
                                </div>
                                <div class="ld-dashboard-instructor-card-body">
                                    <h4 class="ld-dashboard-instructor-name">
                                        <?php
                                        // Allow developers to filter the instructor name.
                                        echo apply_filters( 'ld_dashboard_instructor_name', esc_html( $instructor->display_name ), $instructor );
                                        ?>
                                    </h4>
                                    <div class="ld-dashboard-instructor-course-count">
                                        <span class="ld-dashboard-ins-course-count">
                                            <?php echo esc_html( count( $instructor_courses ) ); ?>
                                        </span>
                                        <span class="ld-dashboard-ins-course-text">
                                            <?php echo esc_html( LearnDash_Custom_Label::get_label( 'courses' ) ); ?>
                                        </span>
                                    </div>
                                    <?php
                                    // Allow developers to add custom content inside the instructor card.
                                    do_action( 'ld_dashboard_instructor_additional_content', $instructor );
                                    ?>
                                </div>
                            </a>
                        </div>
                        <?php
                    }
                } else {
                    ?>
                    <p class="ld-dashboard-warning"><?php echo esc_html__( 'No instructors found.', 'ld-dashboard' ); ?></p>
                    <?php
                }
                ?>
            </div>
        </div>
        <?php
    } else {
        wp_enqueue_script( 'ld-dashboard-shortcodes' );
        $localize_data = array(
            'ajaxurl'            => admin_url( 'admin-ajax.php' ),
            'ajax_nonce'         => wp_create_nonce( 'ajax-nonce' ),
            'nonce'              => wp_create_nonce( 'ld-dashboard' ),
        );

        wp_localize_script( 'ld-dashboard-shortcodes', 'learndash_admin_shortcodes_assets', $localize_data );

        // Load the single instructor template if the instructor is found.
        $template = Ld_Dashboard_Public::template_override_exists( 'ld-dashboard-single-instructor.php' );

        if ( $template ) {
            do_action( 'ld_dashboard_before_single_instructor', $instructor );
            include $template;
            do_action( 'ld_dashboard_after_single_instructor', $instructor );
        } else {
            include LD_DASHBOARD_PLUGIN_DIR . 'templates/ld-dashboard-single-instructor.php';
        }
    }

    return ob_get_clean();
}

add_shortcode( 'ld_dashboard_instructors_list', 'ld_dashboard_instructors_callback' );

/**
 * Updates the rewrite rules for the LearnDash Dashboard instructor listing page.
 *
 * Dynamically updates the rewrite rules when the "Instructors page" setting is changed.
 *
 * @param string $option_name The name of the option that was updated.
 * @param mixed  $old_value   The old option value.
 * @param mixed  $value       The new option value.
 */
function ld_dashboard_update_rewrite_rule( $option_name, $old_value, $value ) {
    if ( 'ld_dashboard_general_settings' !== $option_name ) {
        return;
    }

    if ( empty( $old_value['instructor_listing_page'] ) || empty( $value['instructor_listing_page'] ) ) {
        return;
    }

    if ( $old_value['instructor_listing_page'] === $value['instructor_listing_page'] ) {
        return;
    }

    $instructor_page_id = (int) $value['instructor_listing_page'];
    $page_data = get_post( $instructor_page_id );

    if ( ! is_object( $page_data ) || 'publish' !== $page_data->post_status ) {
        return;
    }

    add_rewrite_tag( '%instructor_username%', '([^/]+)' );

    $rewrite_rule = '^' . $page_data->post_name . '/([^/]+)/?$';
    add_rewrite_rule(
        $rewrite_rule,
        'index.php?pagename=' . $page_data->post_name . '&instructor_username=$matches[1]',
        'top'
    );

    flush_rewrite_rules( false );

    error_log('Rewrite rule added: ' . $rewrite_rule);
}

add_action( 'updated_option', 'ld_dashboard_update_rewrite_rule', 10, 3 );

/**
 * Add rewrite rules and flush them (for debugging only).
 */
add_action('init', function() {
    // Get the page ID for the instructors page from settings.
    $function_obj = class_exists('Ld_Dashboard_Functions') ? Ld_Dashboard_Functions::instance() : null;
    $instructors_page_id = $function_obj ? $function_obj->ld_dashboard_get_page_id('instructors') : null;

    if ($instructors_page_id) {
        $page_slug = get_post_field('post_name', $instructors_page_id);

        // Add rewrite tag and rule for instructor_username.
        add_rewrite_tag('%instructor_username%', '([^/]+)');
        add_rewrite_rule(
            '^' . $page_slug . '/([^/]+)/?$',
            'index.php?pagename=' . $page_slug . '&instructor_username=$matches[1]',
            'top'
        );
    }
}, 10);

// Register query variable for instructor_username.
add_filter('query_vars', function ($vars) {
    $vars[] = 'instructor_username';
    return $vars;
});

// Flush rewrite rules only on plugin activation or settings update.
add_action('updated_option', function($option_name, $old_value, $value) {
    if ($option_name === 'ld_dashboard_general_settings') {
        flush_rewrite_rules();
    }
}, 10, 3);



