<?php

namespace LearnDash\Easy_Digital_Downloads\Tests;

use EDD_Payment;
use WP_UnitTestCase;

/**
 * PaymentTest
 * 
 * @uses WP_UnitTestCase $this
 */
class PaymentTest extends WP_UnitTestCase {
    /**
     * @test
     */
    public function it_gives_user_access_on_payment_completion_with_registered_user()
    {
        // Create EDD product
        $edd_product = $this->factory()->post->create_and_get([
            'post_type' => 'download',
            'post_status' => 'publish',
        ]);
        
        // Create LD course and group
        $ld_course = $this->factory()->post->create_and_get([
            'post_type' => 'sfwd-courses',
            'post_status' => 'publish',
        ]);
        
        $ld_group = $this->factory()->post->create_and_get([
            'post_type' => 'groups',
            'post_status' => 'publish',
        ]);

        // Associate the LD course and group to the EDD product
        add_post_meta( $edd_product->ID, '_edd_learndash_course', [ $ld_course->ID ] );
        add_post_meta( $edd_product->ID, '_edd_learndash_group', [ $ld_group->ID ] );

        // Create user
        $user = $this->factory()->user->create_and_get([
            'role' => 'subscriber',
            'user_email' => 'test.1@localhost.test',
        ]);

        // Make EDD purchase with the user with complete payment
        $order_id = edd_insert_payment([
            'price'        => '10',
            'date'         => date( 'Y-m-d H:i:s' ),
            'user_email'   => $user->user_email,
            'purchase_key' => md5( time() ),
            'currency'     => 'USD',
            'downloads'    => [
                [
                    'id' => $edd_product->ID,
                    'options' => [],
                    'quantity' => 1,
                ]
            ],
            'user_info'    => [
                'id' => $user->ID,
                'email' => $user->user_email,
                'first_name' => $user->first_name,
                'last_name' => $user->last_name,
            ],
            'cart_details' => [
                [
                    'name' => $edd_product->post_title,
                    'id' => $edd_product->ID,
                    'item_number' => array
                        (
                            'id' => $edd_product->ID,
                            'options' => array(),
                            'quantity' => 1,
                        ),
                    'item_price' => '10',
                    'quantity' => 1,
                    'discount' => 0,
                    'subtotal' => '10',
                    'tax' => 0,
                    'fees' => array(),
                    'price' => '10',
                ]
            ],
            'status'       => 'pending',
        ]);

        $edd_payment = new EDD_Payment( $order_id );
        $edd_payment->update_status( 'completed' );
        $edd_payment->save();

        // Assert user course and group access
        $has_course_access = sfwd_lms_has_access( $ld_course->ID, $user->ID );
        $has_group_access = (bool) learndash_is_user_in_group( $user->ID, $ld_group->ID );

        $this->assertTrue( $has_course_access );
        $this->assertTrue( $has_group_access );
    }

    /**
     * @test
     */
    public function it_gives_user_access_on_payment_completion_with_unregistered_user()
    {
        // Create EDD product
        $edd_product = $this->factory()->post->create_and_get([
            'post_type' => 'download',
            'post_status' => 'publish',
        ]);
        
        // Create LD course and group
        $ld_course = $this->factory()->post->create_and_get([
            'post_type' => 'sfwd-courses',
            'post_status' => 'publish',
        ]);
        
        $ld_group = $this->factory()->post->create_and_get([
            'post_type' => 'groups',
            'post_status' => 'publish',
        ]);

        // Associate the LD course and group to the EDD product
        add_post_meta( $edd_product->ID, '_edd_learndash_course', [ $ld_course->ID ] );
        add_post_meta( $edd_product->ID, '_edd_learndash_group', [ $ld_group->ID ] );

        // Make EDD purchase with the user with complete payment
        $order_id = edd_insert_payment([
            'price'        => '10',
            'date'         => date( 'Y-m-d H:i:s' ),
            'user_email'   => 'new.user@localhost.test',
            'purchase_key' => md5( time() ),
            'currency'     => 'USD',
            'downloads'    => [
                [
                    'id' => $edd_product->ID,
                    'options' => [],
                    'quantity' => 1,
                ]
            ],
            'user_info'    => [
                'id' => 0,
                'email' => 'new.user@localhost.test',
                'first_name' => 'First',
                'last_name' => 'Last',
            ],
            'cart_details' => [
                [
                    'name' => $edd_product->post_title,
                    'id' => $edd_product->ID,
                    'item_number' => array
                        (
                            'id' => $edd_product->ID,
                            'options' => array(),
                            'quantity' => 1,
                        ),
                    'item_price' => '10',
                    'quantity' => 1,
                    'discount' => 0,
                    'subtotal' => '10',
                    'tax' => 0,
                    'fees' => array(),
                    'price' => '10',
                ]
            ],
            'status'       => 'pending',
        ]);

        $edd_payment = new EDD_Payment( $order_id );
        $edd_payment->update_status( 'completed' );
        $edd_payment->save();

        $new_edd_payment = edd_get_payment( $order_id );

        // Assert user course and group access
        $has_course_access = sfwd_lms_has_access( $ld_course->ID, $new_edd_payment->user_id );
        $has_group_access = (bool) learndash_is_user_in_group( $new_edd_payment->user_id, $ld_group->ID );

        $this->assertTrue( $has_course_access );
        $this->assertTrue( $has_group_access );
    }
}
