<?php

namespace LearnDash\Easy_Digital_Downloads\Tests;

use EDD_Payment;
use EDD_Subscription;
use WP_UnitTestCase;

class SubscriptionTest extends WP_UnitTestCase {
    
    /**
     * Create user access mock from subscription
     */
    private function create_user_access_mock_from_subscription()
    {
        // Create user
        $user = $this->factory()->user->create_and_get([
            'role' => 'subscriber',
            'user_email' => 'user@localhost.test',
            'first_name' => 'First',
            'last_name' => 'Last',
        ]);

        // Create LD course and group
        $course = $this->factory()->post->create_and_get([
            'post_type'   => 'sfwd-courses',
            'post_status' => 'publish',
        ]);

        $group = $this->factory()->post->create_and_get([
            'post_type' => 'groups',
        ]); 

        // Create subscription product with max 2 recurring payments
        $download = new \EDD_Download();
        $created = $download->create([
            'post_status' => 'publish',
            'post_title'  => 'Subscription Product',
        ]);

        if ( $created ) {
            update_post_meta( $download->ID, '_edd_learndash_course', [ $course->ID ] );
            update_post_meta( $download->ID, '_edd_learndash_group', [ $group->ID ] );

            // EDD
            update_post_meta( $download->ID, 'edd_recurring', 'yes' );
            update_post_meta( $download->ID, 'edd_price', '5.00' );
            update_post_meta( $download->ID, 'edd_period', 'day' );
            update_post_meta( $download->ID, 'edd_times', '2' );
        }

        // Create first payment and subscription
        $payment_id = edd_insert_payment([
            'cart_details' => [
                [
                    'id' => $download->ID,
                    'quantity' => 1,
                    'tax' => 0,
                    'price' => '5.00',
                ]
            ],
            'user_info' => [
                'id' => $user->ID,
                'email' => $user->user_email,
                'first_name' => $user->first_name,
                'last_name' => $user->last_name,
            ],
            'currency' => 'USD',
            'purchase_key' => md5(time()),
        ]);

        $subscription = new EDD_Subscription();
        $subscription->create([
            'product_id' => $download->ID,
            'parent_payment_id' => $payment_id,
        ]);

        // Set payment and subscription status to active
        $payment = new EDD_Payment( $payment_id );
        $payment->update_status( 'active' );
        $payment->save();

        $subscription->update([
            'status' => 'active',
        ]);

        $subscription->status = 'active';

        do_action( 'test_learndash_edd_subscription_created', $subscription );

        return compact( 'user', 'course', 'group', 'payment', 'subscription' );
    }

    /**
     * @test
     */
    public function it_remove_user_access_on_subscription_completion()
    {
        add_action( 'test_learndash_edd_subscription_created', function( $subscription ) {
            // Add LD EDD filter hook that handles subscription completion
            add_filter( 'learndash_edd_remove_access_on_subscription_completion', function() {
                return true;
            });

            // Add a second payment and set the subscription to completed
            $subscription->renew();
            $subscription->complete();
        } );

        extract( $this->create_user_access_mock_from_subscription() );

        $has_access = sfwd_lms_has_access( $course->ID, $user->ID );
        $this->assertFalse( $has_access );
    }

    /**
     * @test
     */
    public function it_keeps_user_accesss_on_subscription_completion()
    {
        add_action( 'test_learndash_edd_subscription_created', function( $subscription ) {
            // Add LD EDD filter hook that handles user access removal subscription completion
            add_filter( 'learndash_edd_remove_access_on_subscription_completion', function() {
                return false;
            });

            // Add a second payment and set the subscription to completed
            $subscription->renew();
            $subscription->complete();
        } );

        extract( $this->create_user_access_mock_from_subscription() );

        $has_access = sfwd_lms_has_access( $course->ID, $user->ID );
        $this->assertTrue( $has_access );
    }

    /**
     * @test
     */
    public function it_remove_user_access_on_subscription_expiration()
    {
        add_action( 'test_learndash_edd_subscription_created', function( $subscription ) {
            // Add LD EDD filter hook that handles subscription expiration
            add_filter( 'learndash_edd_remove_access_on_subscription_expiration', function() {
                return true;
            });

            // Add a second payment and set the subscription to expired
            $subscription->renew();
            $subscription->complete();
            $subscription->expire();
        } );

        extract( $this->create_user_access_mock_from_subscription() );

        $has_access = sfwd_lms_has_access( $course->ID, $user->ID );
        $this->assertFalse( $has_access );
    }

    /**
     * @test
     */
    public function it_keeps_user_accesss_on_subscription_expiration()
    {
        add_action( 'test_learndash_edd_subscription_created', function( $subscription ) {
            // Add LD EDD filter hook that handles user access removal subscription expiration
            add_filter( 'learndash_edd_remove_access_on_subscription_expiration', function() {
                return false;
            });

            // Add a second payment and set the subscription to expired
            $subscription->renew();
            $subscription->complete();
            $subscription->expire();
        } );

        extract( $this->create_user_access_mock_from_subscription() );

        $has_access = sfwd_lms_has_access( $course->ID, $user->ID );
        $this->assertTrue( $has_access );
    }
}
