<?php
/**
 * LearnDash Settings Section for the Learner Notes Features Metabox
 *
 * @since 1.0.0
 * @package learndash_notes\admin
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

use LearnDash\Notes\Templates\Admin_Template;

if ( ( class_exists( 'LearnDash_Settings_Section' ) ) && ( ! class_exists( 'LearnDash_Notes_Settings_Section_Features' ) ) ) {

	/**
	 * Class LearnDash Settings Section for the Learner Notes Features Metabox
	 *
	 * @since 1.0.0
	 *
	 * @phpstan-import-type Inline_Field_Args from LearnDash_Notes_Settings_Section
	 */
	class LearnDash_Notes_Settings_Section_Features extends LearnDash_Notes_Settings_Section {

		/**
		 * Holds the default values for each Setting in this Section
		 *
		 * @var array $defaults Setting Defaults
		 */
		public $defaults = array();

		/**
		 * Protected constructor for class
		 *
		 * @since 1.0.0
		 */
		protected function __construct() {

			// This is the 'option_name' key used in the wp_options table.
			$this->setting_option_key = 'learndash_notes_settings';

			// Used within the Settings API to uniquely identify this section.
			$this->settings_section_key = 'learndash_notes_settings';

			// Section label/header.
			$this->settings_section_label = esc_html__( 'Settings', 'learndash_student_notes' );

			$this->defaults = self::get_default_values();

			parent::__construct();
		}

		/**
		 * Set the Option Key for this Class so that the defaults can be properly loaded no matter when it is requested (such as before the Settings Sections are added by LearnDash)
		 *
		 * @param   array $options  Class=>Option Key.
		 *
		 * @access  public
		 * @since   1.0.0
		 * @return  array           Class=>Option
		 */
		public static function set_option_keys( $options ) {

			$options[ __CLASS__ ] = 'learndash_notes_settings';

			return $options;

		}

		/**
		 * Defines the default values to use for the Settings within this Section
		 *
		 * @access  public
		 * @since   1.0.0
		 * @return  array  Key=>Value pairs for default values
		 */
		public static function get_default_values() {

			$defaults = apply_filters(
				'learndash_notes_features_default_values',
				array(
					'ld_student_notes_add_editor_toolbar' => '',
					'ld_student_notes_show_editor_character_count' => '',
					'ld_student_notes_add_notify_instructor' => '',
					'ld_student_notes_auto_include_add_note' => [],
					'ld_student_notes_auto_include_add_note_popup' => [],
					'ld_student_notes_auto_include_notes_list' => [],
					'ld_student_notes_auto_include_all_notes' => [],
					'ld_student_notes_show_private_notes' => '',
				)
			);

			return $defaults;
		}

		/**
		 * Define your fields here, in the same way you would normally do within LearnDash_Settings_Section::load_settings_fields()
		 *
		 * @since 1.0.0
		 *
		 * @return array<string,mixed>
		 */
		public static function get_settings_fields() {
			$template = new Admin_Template();

			$fields = apply_filters(
				'learndash_notes_email_setting_fields',
				array(
					'ld_student_notes_add_editor_toolbar' => [
						'name'              => 'ld_student_notes_add_editor_toolbar',
						'type'              => 'checkbox-switch',
						'label'             => esc_html__( 'Editor Toolbar', 'learndash_student_notes' ),
						'description_after' => esc_html__( 'Display text formatting tools above the New Notes Section text area.', 'learndash_student_notes' ),
						'options'           => [
							'on' => 1,
						],
						'sanitize_callback' => 'sanitize_text_field',
					],
					'ld_student_notes_show_editor_character_count' => [
						'name'              => 'ld_student_notes_show_editor_character_count',
						'type'              => 'checkbox-switch',
						'label'             => esc_html__( 'Character Count', 'learndash_student_notes' ),
						'description_after' => esc_html__( 'Display the number of characters (excluding spaces) contained within the New Notes Section text area.', 'learndash_student_notes' ),
						'options'           => [
							'on' => 1,
						],
						'sanitize_callback' => 'sanitize_text_field',
					],
					'ld_student_notes_add_notify_instructor' => [
						'name'              => 'ld_student_notes_add_notify_instructor',
						'type'              => 'checkbox-switch',
						'label'             => esc_html__( 'Instructor Notification Emails', 'learndash_student_notes' ),
						'description_after' => wp_kses_post(
							__( 'Send Email notifications to Instructors when Students select the "Notify Instructor" checkbox during submission. Enabling this setting will add the "Notify Instructor" checkbox to the Student Notes submission form.', 'learndash_student_notes' ) .
							'<br /><br />' .
							sprintf(
								// translators: placeholders: Group label, Anchor Link opening and closing tags for documentation.
								__( 'IMPORTANT: Email settings and %1$s settings are necessary for this feature to work. %2$sSet up Instructor Notification Emails now.%3$s', 'learndash_student_notes' ),
								learndash_get_custom_label_lower( 'group' ),
								'<a href="https://go.learndash.com/notesnotificationemail" target="_blank">',
								'</a>'
							)
						),
						'options'           => [
							'on' => 1,
						],
						'sanitize_callback' => 'sanitize_text_field',
					],
					'ld_student_notes_show_private_notes' => [
						'name'              => 'ld_student_notes_show_private_notes',
						'type'              => 'radio',
						'label'             => esc_html__( 'Instructor View Settings', 'learndash_student_notes' ),
						'options'           => [
							'on'   => [
								'label'       => esc_html__( 'Show All Student Notes to Instructors', 'learndash_student_notes' ),
								'description' => esc_html__( 'The admin view of the Notes pages will display all Student Notes to their Instructor(s) / Group Leaders.', 'learndash_student_notes' ),
							],
							'hide' => [
								'label'       => esc_html__( 'Hide all Student Notes from Instructors', 'learndash_student_notes' ),
								'description' => esc_html__( 'The admin view of the Notes pages will NOT display any Student Notes to their Instructor(s) / Group Leaders. Any previously sent Notes set to be visible by the Instructor will still be visible.', 'learndash_student_notes' ),
							],
							''     => [ // Left as an empty string for backwards compatibility.
								'label'       => esc_html__( "Student's choice", 'learndash_student_notes' ),
								'description' => wp_kses_post(
									__( 'The admin view of the Notes pages will only display Student Notes to their Instructor(s) / Group Leaders when a Student selects the "Notify Instructor" checkbox during submission.', 'learndash_student_notes' ) .
									'<br /><br />' .
									__( 'This setting is only available when "Instructor Notification Emails" is enabled.', 'learndash_student_notes' )
								),
							],
						],
						'sanitize_callback' => 'sanitize_text_field',
					],
					'ld_student_notes_auto_include_add_note' => [
						'name'              => 'ld_student_notes_auto_include_add_note',
						'type'              => 'checkbox',
						'label'             => esc_html__( 'New Note Section', 'learndash_student_notes' ),
						'help_text'         => wp_kses_post(
							esc_html__( 'Students can add a new note in this form below the page content.', 'learndash_student_notes' ) .
							'<br /><br />' .
							esc_html__( 'To manually insert the form onto a specific piece of content, use the following shortcode:', 'learndash_student_notes' ) .
							'<br /><br />' .
							'<code>[llms_add_new_note]</code>' .
							$template->template(
								'copy-text',
								[
									'text' => '[llms_add_new_note]',
								],
								false
							)
						),
						'description_after' => esc_html__( 'The New Note Section will display within all of the selected content types above.', 'learndash_student_notes' ),
						'options'           => self::get_course_content_toggle_options(),
						'sanitize_callback' => [ self::class, 'sanitize_course_content_toggle_options' ],
					],
					'ld_student_notes_auto_include_add_note_popup' => [
						'name'              => 'ld_student_notes_auto_include_add_note_popup',
						'type'              => 'checkbox',
						'label'             => esc_html__( 'New Note Popup', 'learndash_student_notes' ),
						'help_text'         => wp_kses_post(
							esc_html__( 'Students can add a new note in this flexible form that “pops” open on page load.', 'learndash_student_notes' ) .
							'<br /><br />' .
							esc_html__( 'To manually insert the form onto a specific piece of content, use the following shortcode:', 'learndash_student_notes' ) .
							'<br /><br />' .
							'<code>[llms_add_new_note_popup]</code>' .
							$template->template(
								'copy-text',
								[
									'text' => '[llms_add_new_note_popup]',
								],
								false
							)
						),
						'description_after' => esc_html__( 'The New Note Popup will display within all of the selected content types above.', 'learndash_student_notes' ),
						'options'           => self::get_course_content_toggle_options(),
						'sanitize_callback' => [ self::class, 'sanitize_course_content_toggle_options' ],
					],
					'ld_student_notes_auto_include_notes_list' => [
						'name'              => 'ld_student_notes_auto_include_notes_list',
						'type'              => 'checkbox',
						'label'             => esc_html__( 'List of Notes', 'learndash_student_notes' ),
						'help_text'         => wp_kses_post(
							esc_html__( "This list displays the all Student's notes submitted on that post or page.", 'learndash_student_notes' ) .
							'<br /><br />' .
							esc_html__( 'To manually insert the form onto a specific piece of content, use the following shortcode:', 'learndash_student_notes' ) .
							'<br /><br />' .
							'<code>[llms_notes_list]</code>' .
							$template->template(
								'copy-text',
								[
									'text' => '[llms_notes_list]',
								],
								false
							)
						),
						'description_after' => esc_html__( 'The List of Notes will display within all of the selected content types above.', 'learndash_student_notes' ),
						'options'           => self::get_course_content_toggle_options(),
						'sanitize_callback' => [ self::class, 'sanitize_course_content_toggle_options' ],
					],
					'ld_student_notes_auto_include_all_notes' => [
						'name'              => 'ld_student_notes_auto_include_all_notes',
						'type'              => 'checkbox',
						'label'             => esc_html__( 'Full List of Notes', 'learndash_student_notes' ),
						'help_text'         => wp_kses_post(
							esc_html__( "This list displays the all the Student's notes.", 'learndash_student_notes' ) .
							'<br /><br />' .
							esc_html__( 'To manually insert the form onto a specific piece of content, use the following shortcode:', 'learndash_student_notes' ) .
							'<br /><br />' .
							'<code>[llms_full_notes_list]</code>' .
							$template->template(
								'copy-text',
								[
									'text' => '[llms_full_notes_list]',
								],
								false
							)
						),
						'description_after' => esc_html__( 'The Full List of Notes will display within all of the selected content types above.', 'learndash_student_notes' ),
						'options'           => self::get_course_content_toggle_options(),
						'sanitize_callback' => [ self::class, 'sanitize_course_content_toggle_options' ],
					],
				)
			);

			return $fields;
		}

		/**
		 * Returns options for toggles per-Course Content Type. Meant to be used with Checkbox, Radio, or Select fields.
		 *
		 * @since 1.1.0
		 *
		 * @return array<string, string>
		 */
		protected static function get_course_content_toggle_options(): array {
			return [
				'on'                     => __( 'All', 'learndash_student_notes' ),
				LDLMS_Post_Types::COURSE => learndash_get_custom_label( 'courses' ),
				LDLMS_Post_Types::LESSON => learndash_get_custom_label( 'lessons' ),
				LDLMS_Post_Types::TOPIC  => learndash_get_custom_label( 'topics' ),
				LDLMS_Post_Types::QUIZ   => learndash_get_custom_label( 'quizzes' ),
			];
		}

		/**
		 * Sanitization callback for Course Content Toggle options.
		 *
		 * @since 1.1.0
		 *
		 * @param mixed $value Submitted option value.
		 *
		 * @return string[]
		 */
		public static function sanitize_course_content_toggle_options( $value ): array {
			if ( ! is_array( $value ) ) {
				return [];
			}

			return array_intersect(
				$value,
				array_keys( self::get_course_content_toggle_options() )
			);
		}
	}

	add_action(
		'learndash_settings_sections_init',
		function() {
			LearnDash_Notes_Settings_Section_Features::add_section_instance();
		}
	);

	LearnDash_Notes_Settings_Section_Features::load_hooks();

}
