<?php

/**
 * The admin-specific functionality of the plugin.
 *
 * @link       https://wooninjas.com/
 * @since      1.0.0
 *
 * @package    WN_LD_Quiz_Import_Export
 * @subpackage WN_LD_Quiz_Import_Export/admin
 */

/**
 * The admin-specific functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the admin-specific stylesheet and JavaScript.
 *
 * @package    WN_LD_Quiz_Import_Export
 * @subpackage WN_LD_Quiz_Import_Export/admin
 * @author     WooNinjas <info@wooninjas.com>
 */
class WN_LD_Quiz_Import_Export_Admin {

	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $plugin_name    The ID of this plugin.
	 */
	private $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $version    The current version of this plugin.
	 */
	private $version;

	private $license, $page_tab, $setting_menu_slug, $options;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.0.0
	 * @param      string    $plugin_name       The name of this plugin.
	 * @param      string    $version    The version of this plugin.
	 * @param      string    $license    The version of this plugin.
	 */
	public function __construct( $plugin_name, $version, $license = null ) {

		$this->plugin_name = $plugin_name;
		$this->version = $version;
		$this->license = $license;

		$this->page_tab = ( isset($_GET['tab']) && !empty($_GET['tab']) ) ? $_GET['tab'] : 'license';

		$this->setting_menu_slug = 'wn-ld-quiz-import-export';
		$this->options = WN_LD_Quiz_Import_Export_Helper::get_settings();
	}

	/**
	 * Register the stylesheets for the admin area.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_styles() {

		$setting_page = isset($_GET['page']) ? sanitize_text_field($_GET['page']) : null;
		$setting_page_tab = isset($_GET['tab']) ? sanitize_text_field($_GET['tab']) : null;

		if ( $setting_page === 'wn-ld-quiz-import-export' && in_array($setting_page_tab, ['general','import_quiz']) ) {
			wp_register_style( "{$this->plugin_name}-select2", plugin_dir_url( __FILE__ ) . 'css/select2.min.css', array(), $this->version, 'all' );
		}

		wp_register_style( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'css/wn-ld-quiz-import-export.css', array(), $this->version, 'all' );

		wp_enqueue_style( "fontawesome-ldqie", "https://cdnjs.cloudflare.com/ajax/libs/font-awesome/6.3.0/css/all.min.css", array(), $this->version, false );

		if ( $setting_page === 'wn-ld-quiz-import-export' ) {
			// Font awesome
			wp_enqueue_style( 'ldqie-cie-fontawesome', 'https://fonts.googleapis.com/css?family=Roboto:300,400,500,700&display=swap', array(), $this->version );
		}
	}

	public function print_styles() {

		$setting_page = isset($_GET['page']) ? sanitize_text_field($_GET['page']) : null;
		$setting_page_tab = isset($_GET['tab']) ? sanitize_text_field($_GET['tab']) : null;

		if ( $setting_page === 'wn-ld-quiz-import-export' && in_array($setting_page_tab, ['general', 'import_quiz']) ) {
			wp_enqueue_style("{$this->plugin_name}-select2");
		}

		wp_enqueue_style($this->plugin_name);
	}

	/**
	 * Register the JavaScript for the admin area.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_scripts() {
		$setting_page = isset($_GET['page']) ? sanitize_text_field($_GET['page']) : null;

		if ( $setting_page === 'wn-ld-quiz-import-export' ) {

			if(( !defined( 'LEARNDASH_SELECT2_LIB' ) ) || ( false === LEARNDASH_SELECT2_LIB )) {
				wp_register_script( "{$this->plugin_name}-select2", LEARNDASH_LMS_PLUGIN_URL . 'assets/vendor/select2-jquery/js/select2.full.min.js', array( 'jquery' ), $this->version, true );
			}

			wp_register_script( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'js/wn-ld-quiz-import-export.js', array( 'jquery' ), $this->version, true );

			$data = $this->get_localize_data();

			wp_localize_script( $this->plugin_name, 'WN_LD_QIE_Vars', $data );
			
			if ( $setting_page === 'wn-ld-quiz-import-export' ) {
				
				wp_register_script( $this->plugin_name.'-json', plugin_dir_url( __FILE__ ) . 'js/wn-ld-quiz-import-export-json.js', array( 'jquery' ), $this->version, true );
			
				wp_localize_script( $this->plugin_name.'-json', 'WN_LD_QIE_Vars', $data );
			}
		}
	}

	public function print_scripts() {
		$setting_page = isset($_GET['page']) ? sanitize_text_field($_GET['page']) : null;
		$setting_page_tab = isset($_GET['tab']) ? sanitize_text_field($_GET['tab']) : null;

		if ( $setting_page === 'wn-ld-quiz-import-export' ) {

			if(( !defined( 'LEARNDASH_SELECT2_LIB' ) ) || ( false === LEARNDASH_SELECT2_LIB )) {
				wp_enqueue_script( "{$this->plugin_name}-select2");
			}

			wp_enqueue_script( $this->plugin_name);
		}

		if ( in_array( $setting_page_tab, ['import_json'] ) ) {
			wp_enqueue_script( $this->plugin_name.'-json');
		}
	}

	/**
	 * @return array
	 */
	public function get_localize_data() {
		$data = array(
			'ajaxurl'     => esc_url( admin_url( 'admin-ajax.php' ) ),
			'ajax_nonce'  => wp_create_nonce( 'ld-qie-ajax-nonce' ),
			'debug'       => defined( 'SCRIPT_DEBUG' ) ? true : false,
			'siteURL'     => site_url(),
			'err_msg1'    => __( 'Invalid format! Please upload JSON file only.', 'learndash-course-import-export' ),
			'err_msg2'    => __( 'Error occured! Please check the format of uploaded file.', 'learndash-course-import-export' ),
			'success_msg' => __( 'File Uploaded please visit Quiz Page', 'learndash-course-import-export' ),
		);

		/*$ldqieVars = array(
			'ajax_url' => admin_url( 'admin-ajax.php' ),
			'ajax_nonce' => wp_create_nonce('ld-qie-ajax-nonce')
		);*/

		$ld_qie_quiz_import_options = WN_LD_Quiz_Import_Export_Helper::get_settings();
		$data['allowExistingQuestionImport'] = !empty($ld_qie_quiz_import_options['import_existing_questions']);
		$is_shared_quiz_questions = LearnDash_Settings_Section::get_section_setting( 'LearnDash_Settings_Quizzes_Builder', 'shared_questions' ) === 'yes';
		$data['isSharedQuestionsAllowed'] =  $is_shared_quiz_questions;

		$data['existingQuestionHeaders'][] = learndash_get_custom_label('Question') . ' ' . __('Title');
		$data['existingQuestionHeaders'][] = __('Points');
		$data['existingQuestionHeaders'][] = learndash_get_custom_label('Question') . ' ' . __('Text');
		$data['existingQuestionHeaders'][] = __('Answer Type');
		$data['existingQuestionHeaders'][] = learndash_get_custom_label('Quizzes');

		//wp_localize_script( 'ld-qie-admin-quiz-import-js', 'ldqieVars', $ldqieVars );

		return $data;
	}

	public function upgrade35() {

        $old_settings_option_key = 'quiz_default';
        $new_settings_option_key = 'ld_qie_settings';
        $old_addon_options = get_option($old_settings_option_key);
        $new_addon_options = get_option($new_settings_option_key, false);

        $old_default_options = [
            'DiscardQuizQuestion' => 'no',
            'DiscardOldQuestion' => 'no',
            'minimum_role_to_administer' => 'manage_options',
            'allow_quiz_publish' => 'yes',
            'AllowExistingQuestionImport' => 'no',
            'setting_quiz_post_id' => 0,
            'update_duplicate_questions' => false,
            'activity_log' => false,
            'debug_log' => false
        ];

        $new_default_options = WN_LD_Quiz_Import_Export_Helper::get_settings();

        $default_option_keys = array_combine(array_keys($old_default_options), array_keys($new_default_options));

        if( !is_array($new_addon_options) ) {
            $new_addon_options = $new_default_options;
            if( !empty($old_addon_options) ) {
                foreach ($default_option_keys as $default_option_old_key => $default_option_new_key) {
                    if(isset($old_addon_options[$default_option_old_key]) && !empty($old_addon_options[$default_option_old_key])) {
                        if($old_addon_options[$default_option_old_key] === 'yes') {
                            $new_addon_options[$default_option_new_key] = true;
                        } elseif($old_addon_options[$default_option_old_key] === 'no') {
                            $new_addon_options[$default_option_new_key] = false;
                        } else {
                            $new_addon_options[$default_option_new_key] =  $old_addon_options[$default_option_old_key];
                        }
                    }
                }
            }

            if( $new_addon_options['delete_old_questions']) {
                $new_addon_options['discard_old_questions'] = 'quiz';
                if($new_addon_options['delete_old_questions']) {
                    $new_addon_options['discard_old_questions'] = 'db';
                }
            } else {
                $new_addon_options['discard_old_questions'] = 'no';
            }

            unset($new_addon_options['delete_old_questions']);

            update_option($new_settings_option_key, $new_addon_options);
            delete_option($old_settings_option_key);
        }

        update_option('ldqie_version', $this->version);
	}

	/**
	 * Add plugin action links
	 *
	 * @param $links
	 *
	 * @return mixed
	 */
	public function plugin_action_links( $links ) {
		$settings_link = '<a href="' . admin_url('admin.php?page=' . $this->plugin_name) . '">'. __( 'Settings', 'wn-ld-quiz-import-export' ). '</a>';
		array_unshift($links, $settings_link);
		return $links;
	}

	/**
	 * Add plugin's menu
	 */
	public function admin_menu() {

	    if( !isset($this->options['required_capability']) || empty($this->options['required_capability']) ) {
		    $this->options['required_capability'] = 'manage_options';
        }

		add_submenu_page(
			'learndash-lms',
			__( 'LearnDash Quiz Import/Export', 'wn-ld-quiz-import-export' ),
			__( 'Quiz Import/Export', 'wn-ld-quiz-import-export' ),
			$this->options['required_capability'],
			$this->plugin_name,
			[ $this, 'plugin_page' ]
		);

		if( current_user_can('manage_options') ) {
			return;
		}

		if( current_user_can( $this->options['required_capability'])  ) {
			remove_submenu_page( 'learndash-lms', 'edit.php?post_type=sfwd-courses' );
			remove_submenu_page( 'learndash-lms', 'edit.php?post_type=sfwd-lessons' );
			remove_submenu_page( 'learndash-lms', 'edit.php?post_type=sfwd-topic' );
			remove_submenu_page( 'learndash-lms', 'edit.php?post_type=sfwd-certificates' );
		}
	}

	public function plugin_row_meta($links, $file) {

		if ( plugin_basename(WN_LD_QUIZ_IMPORT_EXPORT_FILE) === $file ) {
			$row_meta = array(
				'docs'    => '<a href="' . esc_url( apply_filters( 'WN_LD_Quiz_Import_Export_docs_url', 'https://docs.wooninjas.com/article/15-learndash-quiz-import-export-overview' ) ) . '" aria-label="' . sprintf( esc_attr__( 'View %s documentation', 'wn-ld-quiz-import-export' ), $this->plugin_name ) . '" target="_blank">' . esc_html__( 'Docs', 'wn-ld-quiz-import-export' ) . '</a>',
				'support' => '<a href="' . esc_url( apply_filters( 'WN_LD_Quiz_Import_Export_support_url', 'https://wooninjas.com/open-support-ticket/' ) ) . '" aria-label="' . esc_attr__( 'Visit premium customer support', 'wn-ld-quiz-import-export' ) . '" target="_blank">' . esc_html__( 'Premium support', 'wn-ld-quiz-import-export' ) . '</a>',
			);

			return array_merge( $links, $row_meta );
		}

		return (array) $links;
	}

	/**
	 * Plugin Review Notice
	 */
	public function plugin_review_notice() {

		if( !current_user_can('manage_options') || ! is_admin() || ! is_plugin_active(  plugin_basename ( WN_LD_QUIZ_IMPORT_EXPORT_FILE ) ) ) {
			return;
		}

		$user_id = get_current_user_id();
		$review_dismissed_key = $this->plugin_name . '_review_dismissed_' . $user_id;
		$review_dismissed_action_key = $this->plugin_name . '_dismiss_notice';

		if( isset( $_GET[$review_dismissed_action_key] ) ) {
			set_transient($review_dismissed_key, 1, MONTH_IN_SECONDS);
		}

		// Show review notice on plugin setting page
		$is_settings_page = ( isset( $_GET['page'] ) && $_GET['page'] == $this->plugin_name );

		if( $is_settings_page ) {
			$user_data = get_userdata( get_current_user_id() );
			$review_dismissed = get_transient($review_dismissed_key);
			$dismiss_url = add_query_arg( $review_dismissed_action_key, 1 );

			if( ! function_exists('get_plugin_data') ){
				require_once( ABSPATH . 'wp-admin/includes/plugin.php' );
			}

			$plugin_data = get_plugin_data( WN_LD_QUIZ_IMPORT_EXPORT_FILE );

			$message = __('Hey %s, Thank you for using <strong>%s</strong>. If you find our plugin useful please take some time to leave a review <a href="%s" target="_blank">here</a>, it will really help us to grow our business.');
			$message = sprintf( $message, esc_html( $user_data->user_nicename ), $plugin_data['Name'], $plugin_data['PluginURI'] );

			$message_html =  sprintf(__('<div class="notice notice-info wn-review-notice" style="padding-right: 38px; position: relative;">
						<p>%s</p>
					<button type="button" class="notice-dismiss"  onclick="location.href=\'%s\';"><span class="screen-reader-text">%s</span></button></div>', 'wn-ld-quiz-import-export'), $message, $dismiss_url, __('Dismiss this notice.', 'wn-ld-quiz-import-export'));

			if ( ! $review_dismissed ) {
				echo $message_html;
			}
		}
	}

	/**
	 * Add branding to footer
	 *
	 * @param $footer_text
	 *
	 * @return mixed
	 */
	function admin_footer_text( $footer_text ) {
		if( isset( $_GET['page'] ) && ( $_GET['page'] == $this->plugin_name ) ) {
			_e('Powered by <a href="http://www.wordpress.org" target="_blank">WordPress</a> | Designed &amp; Developed by <a href="https://wooninjas.com" target="_blank">WooNinjas</a></p>', 'wn-ld-quiz-import-export');
		} else {
			return $footer_text;
		}
	}

	public function plugin_page() {
		$page_tab = ( isset($_GET['tab']) && !empty($_GET['tab']) ) ? $_GET['tab'] : 'import_quiz';

		?>
		<div class="wrap wn_wrap">

			<?php settings_errors(); ?>
			<div id="icon-options-general" class="icon32"></div>
			<h1 class="ld-qei-nav"><?php echo __( 'LearnDash Quiz Import Export', 'wn-ld-quiz-import-export' ); ?></h1>

			<div class="nav-tab-wrapper ld-quiz-ei-nav">
				<?php
				$sections = $this->get_sections();
				$url = admin_url('admin.php?page=' . $this->plugin_name);

				foreach($sections as $key => $section ) {
					$url = add_query_arg('tab', $key, $url);

					if(isset($section['action'])) {
						$url = add_query_arg('action', $section['action'], $url);
					} else {
						remove_query_arg('action', $url);
					}

					if(isset($section['id'])) {
						$url = add_query_arg('id', $section['id'], $url);
					} else {
						remove_query_arg('id', $url);
					}
					?>
					<a href="<?php echo $url; ?>"
					   class="nav-tab <?php echo $page_tab == $key ? 'nav-tab-active' : ''; ?>">
						<i class="dashicons dashicons-<?php echo $section['icon']; ?>" aria-hidden="true"></i>
						<?php echo $section['title']; ?>
					</a>
					<?php
				}
				?>
			</div>

			<?php
			foreach( $sections as $key => $section ) {
				if( $page_tab == $key ) {
					include( 'partials/' . $key . '.php' );
				}
			}
			?>

		</div>
		<?php
	}

	private function get_sections() {
		$admin_sections = array();
		$sections = array();

		$sections = array(
			'import_quiz' => array(
				'title' => __( 'Import Quiz' , 'wn-ld-quiz-import-export'),
				'icon' => 'welcome-add-page',
			),
			'import_json' => array(
				'title' => __( 'Import JSON' , 'wn-ld-quiz-import-export'),
				'icon' => 'welcome-add-page',
			),
		);


		if( $this->options['activity_log'] === true ) {
			$sections['activity_log'] = array(
				'title' => __( 'Activity Log', 'wn-ld-quiz-import-export' ),
				'icon' => 'warning',
			);
		}
		
		if( $this->options['debug_log'] === true ) {
			$sections['debug_log'] = array(
				'title' => __( 'Debug Log', 'wn-ld-quiz-import-export' ),
				'icon' => 'warning',
			);
		}

		

		if( current_user_can('manage_options') ) {

			$admin_sections = array(
				'license' => array(
					'title' => __( 'License', 'wn-ld-quiz-import-export' ),
					'icon' => 'update',
				),
				'general' => array(
					'title' => __( 'General Settings', 'wn-ld-quiz-import-export' ),
					'icon' => 'admin-settings',
				),
				'quiz_settings' => array(
					'title' => __( 'Quiz Settings', 'wn-ld-quiz-import-export' ),
					'icon' => 'admin-settings',
				),
			);
		}

		return array_merge( $admin_sections, $sections );
	}

	/**
	 * Adds the extra capabilities to manage the quiz/question
	 *
	 * @param $allcaps
	 *
	 * @return $allcaps
	 */
	function ld_add_extra_caps($allcaps, $cap, $args){

		$values = WN_LD_Quiz_Import_Export_Helper::get_settings();
		if( isset( $values['required_capability'] ) && !empty( trim( $values['required_capability'] ) ) && trim( $values['required_capability'] ) != 'manage_options' ) {
			$user = new WP_User( get_current_user_id() );
			if( ! in_array( 'administrator', $user->roles ) ) {
				$editor_role_test = trim( $values['required_capability'] );
				if( trim( $values['required_capability'] ) == 'publish_posts' ) {
					$editor_role_test = 'delete_others_posts';
				}

				if( array_key_exists( $values['required_capability'], $allcaps ) || array_key_exists( $editor_role_test, $allcaps )  ) {
					$allcaps[ 'read_course' ] = 1;
					if( isset( $values['publish_quiz_on_import'] ) && $values["publish_quiz_on_import"] != "no" ) {
						$allcaps['publish_courses'] = 1;
					}
					$allcaps[ 'edit_published_courses' ] = 1;
					$allcaps[ 'delete_published_courses' ] = 1;
					$allcaps[ 'edit_courses' ] = 1;
					$allcaps[ 'wpProQuiz_add_quiz' ] = 1;
					$allcaps[ 'wpProQuiz_edit_quiz' ] = 1;
					$allcaps[ 'wpProQuiz_delete_quiz' ] = 1;
					$allcaps[ 'wpProQuiz_show' ] = 1;
				}
			}
		}

		return $allcaps;
	}

	/**
	 * Save Settings
	 */
	public function save_settings() {

		if ( isset($_POST['ld_qie_settings_general_field']) && check_admin_referer('ld_qie_settings_general','ld_qie_settings_general_field') ) {
			$saved_settings = WN_LD_Quiz_Import_Export_Helper::get_settings();
			$post_settings = [];
			if( isset($_POST['ld_qie_settings']) && is_array($_POST['ld_qie_settings']) ) {
				$post_settings = $_POST['ld_qie_settings'];
			}
			$settings = [];

			foreach ($saved_settings as $setting_key => $setting_value) {

				if( current_user_can('manage_options') || !in_array($setting_key, ['required_capability', 'publish_quiz_on_import']) ) {

					if ( is_bool( $setting_value ) ) {
						$setting_value = isset( $post_settings[ $setting_key ] );
					} else {
						if ( isset( $post_settings[ $setting_key ] ) ) {
							$setting_value = sanitize_text_field( $post_settings[ $setting_key ] );
						}
					}
				}

				$settings[ $setting_key ] = $setting_value;
			}

			update_option('ld_qie_settings', $settings);

			$url = admin_url('admin.php');
			$url = add_query_arg('page', $this->plugin_name, $url);
			$url = add_query_arg('tab', $this->page_tab, $url);
			$url = add_query_arg('ld_qie_settings_saved', time(), $url);

			wp_safe_redirect($url);
		}
	}

	/**
	 * Setting Save Notice
	 */
	public function notice_settings_saved() {

		if(!isset($_GET['ld_qie_settings_saved'])) return;

		?>
        <div class="notice notice-success is-dismissible">
            <p><strong><?php _e( 'Settings saved.', 'wn-ld-quiz-import-export' ); ?></strong></p>
        </div>
		<?php
	}

	public function upgrade() {
		$plugin_version = preg_replace( '/[^0-9.].*/', '', get_option( 'ldqie_version' ) );
		if( version_compare( $plugin_version, '3.7', '<' ) ) {
			$this->upgrade37();
		}
	}

	private function upgrade37() {
		$plugin_version = preg_replace( '/[^0-9.].*/', '', get_option( 'ldqie_version' ) );
		if( version_compare( $plugin_version, '3.5', '<' ) ) {
			$this->upgrade35();
		}

		$plugin_option_prefix = 'ld_qie';

		$old_option_key = 'wn_ldqie_license_key';
		$license_key = trim(get_option($old_option_key));
		if( (bool) $license_key !== false ) {
			update_option( "{$plugin_option_prefix}_license_key", $license_key );
			delete_option($old_option_key);
		}

		$old_option_key = 'ldqie_version';
		$version = trim(get_option($old_option_key));
		if( (bool) $version !== false ) {
			update_option( "{$plugin_option_prefix}_version", $version );
			delete_option($old_option_key);
		}

		$old_option_key = 'edd_learndash_quiz_import_export_license_active';
		$license_status = trim(get_option($old_option_key));
		if( (bool) $license_status !== false ) {
			update_option( "{$plugin_option_prefix}_license_status", $license_status );
			delete_option($old_option_key);
		}

		update_option("{$plugin_option_prefix}_version", $this->version);
	}

	public function ld_qie_upload_json_file_cb() {
		
		$file_formats = [ 'json' ]; 
		
		$file = isset( $_FILES['ldqie_import_file'] ) ? $_FILES['ldqie_import_file'] : null;
		if ( ! empty( $file ) ) {
			$ext = strtolower( pathinfo( sanitize_text_field( $_FILES['ldqie_import_file']['name'] ), PATHINFO_EXTENSION ) );
			if ( in_array( $ext, $file_formats ) ) {
				global $current_user;
				$upload_dir   = wp_upload_dir();
				$user_dirname = $upload_dir['basedir'] . '/ld_qie_uploads/' . $current_user->ID;
				if ( ! file_exists( $user_dirname ) ) {
					wp_mkdir_p( $user_dirname );
				}

				$file_path = trailingslashit( $user_dirname ) . $file['name'];
				move_uploaded_file( $file["tmp_name"], $file_path );

				$this->file_loaded = true;
				$this->start_import( $file_path, $ext );
			}
			return 1; //Invalid file format
		}
		return 0; //Invalid file
	}

	private function start_import( $file_path, $ext ) {
		$json_importer = new WN_LD_Quiz_Import_Export_Json_Importer();
		$d = $json_importer->saveImport( file_get_contents( $file_path ) );
		$data = array( 'success' => true, 'c_ids' => file_get_contents( $file_path ) );
		wp_send_json( $data, 200 );

	}

	public function fix_taxonomy_childrens() {
		$taxonomy = 'multi_question_category';
		if ( ! is_taxonomy_hierarchical( $taxonomy ) ) {
			return array();
		}

		$children = get_option( "{$taxonomy}_children" );

		if ( is_array( $children ) ) {
			return $children;
		}
		$children = array();
		$terms    = get_terms(
			array(
				'taxonomy'               => $taxonomy,
				'get'                    => 'all',
				'orderby'                => 'id',
				'fields'                 => 'id=>parent',
				'update_term_meta_cache' => false,
			)
		);
		foreach ( $terms as $term_id => $parent ) {
			if ( $parent > 0 ) {
				$children[ $parent ][] = $term_id;
			}
		}
		update_option( "{$taxonomy}_children", $children );
	}

	public function save_quiz_settings() {

		if( ! current_user_can( 'manage_options' ) ) {
			return;
		}

		// Check if we are on plugin settings page.
		if ( isset( $_REQUEST['page'] ) && 'wn-ld-quiz-import-export' === rtrim( $_REQUEST['page'] ) ) {

			// Check if active tab is quiz_settings
			if( isset( $_REQUEST['tab'] ) && 'quiz_settings' === rtrim( $_REQUEST['tab'] ) ) {

				if( ! empty( $_POST ) ) { 
					
					$ldqie_nonce = isset( $_POST['ldqie_quiz_settings_nonce'] ) ? $_POST['ldqie_quiz_settings_nonce'] : -100;

					// Check if nonce is valid
					if( ! wp_verify_nonce( $ldqie_nonce, 'ldqie_quiz_settings_nonce' ) ) {
						 die( 'Process stopped, request could not be verified please contact administrator.' );
					}

					$ldqie_settings = get_option( '__ldqie_plugin_quiz_settings', array() );
					
					$ldqie_settings['ldqie_allowed_users'] 		 	= isset( $_POST['ldqie-allowed-users'] ) ? $_POST['ldqie-allowed-users'] : 0; 	
					$ldqie_settings['ldqie_question_completion']	= isset( $_POST['ldqie-question-completion'] ) ? $_POST['ldqie-question-completion'] : 0; 	
					$ldqie_settings['ldqie_question_number'] 		= isset( $_POST['ldqie-question-number'] ) ? $_POST['ldqie-question-number'] : 0; 	
					$ldqie_settings['ldqie_quiz_autostart'] 		= isset( $_POST['ldqie-quiz-autostart'] ) ? $_POST['ldqie-quiz-autostart'] : 0; 	
					
					$ldqie_settings['ldqie_question_display'] 		= isset( $_POST['ldqie-question-display'] ) ? $_POST['ldqie-question-display'] : 0;

					if( 'multiple' === $ldqie_settings['ldqie_question_display'] ) {
						$ldqie_settings['ldqie_question_per_page'] 	= isset( $_POST['ldqie-question-per-page'] ) ? $_POST['ldqie-question-per-page'] : 0;
					} elseif( 'single' === $ldqie_settings['ldqie_question_display'] ) {
						$ldqie_settings['ldqie_question_display_result'] 	= isset( $_POST['ldqie-question-display-result'] ) ? $_POST['ldqie-question-display-result'] : 0;
					}

					$ldqie_settings['ldqie_question_overview'] 		= isset( $_POST['ldqie-question-overview'] ) ? $_POST['ldqie-question-overview'] : 0; 	

					// Update ldqie global settings.
					update_option( '__ldqie_plugin_quiz_settings', $ldqie_settings );
				} 
			}
		}
	}

	
}
