<?php
/**
 * LearnDash Quiz Import/Export Add-on Importer
 */

defined('ABSPATH') || exit;

use PhpOffice\PhpSpreadsheet\IOFactory;
use PhpOffice\PhpSpreadsheet\Cell\Coordinate;

if( !class_exists('WN_LD_Quiz_Import_Export_Importer') ) {

	/**
	 * Class WN_LD_Quiz_Import_Export_Importer
	 */
	class WN_LD_Quiz_Import_Export_Importer {

		/** @var PhpOffice\PhpSpreadsheet\Spreadsheet */
		var $phpExcel;
		var $file_loaded = false;
		var $worksheets;
		var $settings = [];
		var $file_path_transient_key = 'ld_qie_file_path';
		var $quiz_data_transient_key = 'ld_qie_quiz_data';
		var $question_quizzes_transient_key = 'ld_qie_question_quizzes';
		var $current_file_path = null;
		var $debug_log_file = null;

		/**
		 * Class Constructor
		 */
		public function __construct() {

			$this->current_file_path = get_transient($this->file_path_transient_key);
			if (empty($this->current_file_path)) {
				$this->current_file_path = get_option($this->file_path_transient_key, '');
			}

			$this->settings = WN_LD_Quiz_Import_Export_Helper::get_settings();

			$upload_dir = wp_upload_dir( null, false );
			$debug_log_filename = 'ld-qie-debug.log';
			$this->debug_log_file = trailingslashit( $upload_dir['basedir'] ) . $debug_log_filename;

			/** ACTIVITY LOG **/
			if( $this->settings['activity_log'] === true ) {
				$this->set_custom_debug_log();
			} else {
				$this->delete_debug_log_file();
			}
		}

		private function delete_debug_log_file() {
			if(file_exists($this->debug_log_file)) {
				unlink( $this->debug_log_file );
			}
		}

		public function clear_debug_log() {

			$page = isset($_GET['page']) ? sanitize_text_field($_GET['page']) : '';
			$tab = isset($_GET['tab']) ? sanitize_text_field($_GET['tab']) : '';
			$action = isset($_GET['action']) ? sanitize_text_field($_GET['action']) : '';

			if( $page === 'wn-ld-quiz-import-export' && $tab === 'activity_log' && $action === 'clear_debug_log' ) {

				$this->delete_debug_log_file();

				wp_safe_redirect(add_query_arg( array(
					'page' => 'wn-ld-quiz-import-export',
					'tab' => 'activity_log',
				), admin_url( 'admin.php' ) ));
			}
		}

		private function set_custom_debug_log() {

			$action = isset($_POST['action']) ? sanitize_text_field($_POST['action']) : '';

			if ( isset( $_POST ) && isset( $_POST['action'] ) && in_array( $action, [
					'ld_qie_submit_quizzes_data',
					'ld_qie_get_quizzes_list',
					'ld_qie_process_quiz_import'
				] ) ) {

				// date_default_timezone_set( get_option( 'timezone_string', 'UTC' ) );
				set_error_handler( [ $this, 'error_handler' ] );
				add_action( 'shutdown', [ $this, 'shutdown_handler' ] );

				ini_set( 'log_errors', 1 );
				ini_set( 'error_log', $this->debug_log_file );
			} else {
				restore_error_handler();
			}
		}

		public function shutdown_handler($array) {

			$error = error_get_last();

			if( empty($error) || (isset($error['file']) && !empty($error['file'])  && $error['file'] != 'Unknown' && strpos('learndash-quiz-import-export', $error[ 'file']) === false) ) {
				return false;
			}

			( new WN_LD_Quiz_Import_Export_Logging() )->log( $error['message'] );
			exit();
		}

		public function error_handler($errno, $errstr, $errfile, $errline) {
			if( strpos('learndash-quiz-import-export', $errfile) !== false ) {


				WN_LD_Quiz_Import_Export_Helper::debug_log( $errfile, false, true );
				$errstr = esc_attr( $errstr );
				( new WN_LD_Quiz_Import_Export_Logging() )->log( 'error-' . $errstr );
				if ( isset( $_POST ) && isset( $_POST['action'] ) && in_array( $_POST['action'], [
						'ld_qie_submit_quizzes_data',
						'ld_qie_get_quizzes_list',
						'ld_qie_process_quiz_import'
					] ) ) {

					return true;
				}
			}

			/*var_dump($errstr);
			WN_LD_Quiz_Import_Export_Helper::debug_log($errstr, false, true);*/

			//throw new Exception($errstr, $errno);

			/* Don't execute PHP internal error handler */
			//return true;

			return true;
		}

		private function log_message($message) {
			if( $this->settings['activity_log'] === true ) {
				WN_LD_Quiz_Import_Export_Helper::debug_log( $message );
			}
		}

		public function import_quiz_cb() {

			if ( ! wp_doing_ajax() ) {
				return;
			}

			if( !check_ajax_referer('ld-qie-ajax-nonce', 'ld_qie_ajax_nonce')) {
				return;
			}

			$this->log_message('Import start...');

			delete_transient($this->quiz_data_transient_key);
			delete_transient($this->question_quizzes_transient_key);

			if( !empty($_POST) && isset($_POST['ld_qie_quiz_data']) ) {

				if( is_array($_POST['ld_qie_quiz_data']) && !empty($_POST['ld_qie_quiz_data']) ) {

					$quiz_data = $_POST['ld_qie_quiz_data'];

					set_transient( $this->quiz_data_transient_key, $quiz_data, HOUR_IN_SECONDS );
					if( isset($_POST['ld_qie_question_quizzes']) && is_array($_POST['ld_qie_question_quizzes']) && !empty($_POST['ld_qie_question_quizzes']) ) {
						$question_quizzes = $_POST['ld_qie_question_quizzes'];

						set_transient( $this->question_quizzes_transient_key, $question_quizzes, HOUR_IN_SECONDS );
					}
					$this->log_message('Import data submitted successfully', 'wn-ld-quiz-import-export');
					$this->send_json([
						'status' => 'success'
					]);
				}
			}
			$this->log_message('Import data submission unsuccessful', 'wn-ld-quiz-import-export');
			restore_error_handler();
		}


		public function ldqie_import_multi_categories( $categories, $question_title, $question_post_id ) {
			WN_LD_Quiz_Import_Export_Helper::multi_question_category_parse_categories_field( $categories, $question_title, $question_post_id );
		}
	
		public function process_import() {

			if ( ! wp_doing_ajax() ) {
				return;
			}

			if( !check_ajax_referer('ld-qie-ajax-nonce', 'ld_qie_ajax_nonce')) {
				return;
			}

			//Increase memory limit and execution time to avoid timeouts
			ini_set('memory_limit ', '128M');
			ini_set('max_execution_time', 15 * MINUTE_IN_SECONDS);
			ini_set('max_input_time', 15 * MINUTE_IN_SECONDS);
			ini_set('max_input_vars', 100 * 1000);

			$next_sheet_index = absint(@$_POST['next_sheet_index']);

			$response = array(
				'status'  => 'error',
				'message' => __( 'An error occurred while processing your request', 'wn-ld-quiz-import-export' )
			);

			$loaded = $this->load_all_worksheets();

			if ( $loaded && ! empty( $this->worksheets ) ) {
				$this->log_message('Worksheets loaded successfully', 'wn-ld-quiz-import-export');
				$total = count($this->worksheets);
				if(empty($next_sheet_index)) {
					$current = 0;
				} else {
					$current = $next_sheet_index;
				}

				if( isset($this->worksheets[$current]) ) {
					$this->log_message(sprintf(__( 'Processing current worksheet #%s','wn-ld-quiz-import-export'), $current));
					$quizzes_form_data = get_transient( $this->quiz_data_transient_key );
					$questions_form_data = get_transient( $this->question_quizzes_transient_key );

					$worksheet = $this->worksheets[$current];

					$form_quiz_data = $quizzes_form_data[$worksheet];

					/** set the current active worksheet by name **/
					$this->phpExcel->setActiveSheetIndexByName( $worksheet );
					$activeSheet = $this->phpExcel->getActiveSheet();
					$sheet = $activeSheet->toArray( null, true, true, true );
					$header_row = [];
					$quiz_post_id = 0;

					//Plugin setting vars
					$unattach_existing_questions = $this->settings['discard_old_questions'] !== 'no';
					$delete_existing_questions = $this->settings['discard_old_questions'] === 'db';
					
					// Flag to skip updating question if already exists in quiz
					$skip_update = !empty( $this->settings['update_duplicate_questions'] ) ? true : false;
					
					// $skip_update = $unattach_existing_questions || $this->settings['update_duplicate_questions'] === false; //Flag to skip updating question if already exists in quiz

					// Suspend bunches of stuff in WP core
					wp_defer_term_counting( true );
					wp_defer_comment_counting( true );
					wp_suspend_cache_invalidation( true );

					//global $wpdb;
					//$wpdb->query( 'SET autocommit = 0;' );

					foreach ( $sheet as $row => &$columns ) {

						if ( $row === 1 ) {
							foreach ( $columns as $column ) {
								$col_name     = WN_LD_Quiz_Import_Export_Helper::string_replace_spaces_to_underscores($column); //convert column name spaces to underscores
								$header_row[] = $col_name;
							}
						} else {

							$columns = array_combine( $header_row, array_values( $columns ) );

							if ( $row === 2 ) { //get quiz columns from second row

								$quiz_data          = [];
								$quiz_data['_name'] = $columns['quiz_title'];
								$quiz_data['_quiz_content']   = @$columns['quiz_content'];
								$quiz_data['_quiz_content']   = @$columns['quiz_content'];
								$quiz_data['_quiz_category']  = @$columns['quiz_category'];
								$quiz_data['_quiz_tags']  	  = @$columns['quiz_tags'];
								$quiz_data['_quiz_material']  = @$columns['materials'];

								// elementor support
								$quiz_data = array_merge($quiz_data, array(
									'_elementor_edit_mode'         => @$columns['elementor_edit_mode'],
									'_elementor_template_type'     => @$columns['elementor_template_type'],
									'_elementor_version'           => @$columns['elementor_version'],
									'_elementor_pro_version'       => @$columns['elementor_pro_version'],
									'_elementor_page_assets'       => @$columns['elementor_page_assets'],
									'_elementor_data'              => @$columns['elementor_data'],
									'_elementor_css'               => @$columns['elementor_css']
								));

								/**
								 * Filter hook to change setting quiz post ID
								 */
								$setting_quiz_post_id = apply_filters('ld_qie_setting_quiz_post_id', absint($this->settings['setting_quiz_post_id']), $quiz_data, $form_quiz_data);

								/**
								 * Action hook before quiz import
								 */
								do_action('ld_qie_before_quiz_import', $quiz_data, $form_quiz_data);

								$quiz_post_id = $this->saveQuiz( $quiz_data, $setting_quiz_post_id, $form_quiz_data );

								/**
								 * Action hook after quiz import
								 */
								do_action('ld_qie_after_quiz_import', $quiz_data, $form_quiz_data, $quiz_post_id);

								$quiz_questions = get_post_meta( $quiz_post_id, 'ld_quiz_questions', true );

								if( !is_array($quiz_questions) ) {
									$quiz_questions = [];
								}

								$quiz_questions_to_delete = $quiz_questions;

								if($unattach_existing_questions) {
									$quiz_questions = [];

									if($delete_existing_questions) {

										$questionMapper = new WpProQuiz_Model_QuestionMapper();

										foreach ($quiz_questions_to_delete as $question_post_id => $question_pro_id) {

											$quiz_ids = learndash_get_quizzes_for_question($question_post_id, true);

											if( count($quiz_ids) <= 1 && in_array($quiz_post_id, array_keys($quiz_ids)) ) {
												wp_delete_post($question_post_id);
												$questionMapper->delete($question_pro_id);
											}
										}
									}
								}
							}

							//Skip advance quizzes questions import when LearnDash Advanced Quizzes add-on is not activated
							$learndash_advanced_quiz_question_types = [
								'laq_jumbled_sentence',
								'laq_swipe_questions',
								'laq_calculated_formula',
								'laq_jumble_dnd_sentence',
								'laq_audio_questions'
							];

							if ( !class_exists('Learndash_Advanced_Quiz') && in_array($columns['question'], $learndash_advanced_quiz_question_types) ) {
								continue;
							}

							/**
							 * Filter hook to modify question data
							 */
							$columns = apply_filters('ld_qie_question_data', $columns, $quiz_post_id);

							$columns['question'] = strtolower( sanitize_text_field( $columns['question'] ) );

							$is_shared_question = false;
							$shared_questions_setting = LearnDash_Settings_Section::get_section_setting('LearnDash_Settings_Quizzes_Builder', 'shared_questions');
						    if ( $shared_questions_setting === 'yes' ) {
						        $is_shared_question = true;
						    }

							$question = $this->getQuestionModel( $columns, $is_shared_question );

							if ( ! empty( $question ) ) {

								/**
								 * Action hook before question import
								 */
								do_action('ld_qie_before_question_import', $question, $quiz_post_id, $is_shared_question);

								list($question_post_id, $question_pro_id) = $this->saveQuestion( $question, $quiz_post_id, $is_shared_question, $skip_update );

								if( $question_post_id !== 0 ) {

									/**
									 * Action hook after question import
									 */
									do_action( 'ld_qie_after_question_import', $question, $quiz_post_id, $is_shared_question, $question_post_id );

									$this->save_advanced_quizzes_question_meta( $question_post_id, $columns );
									$this->update_advanced_quizzes_question_settings( $question_post_id, $columns );

									$quiz_questions[ $question_post_id ] = $question_pro_id;
									
									// Import multi question category
									$this->ldqie_import_multi_categories( 
										@$columns['multi_question_category'], 
										@$columns['title'], 
										$question_post_id 
									);

								}
							}
						}
					}

					//$wpdb->query( 'COMMIT;' );

					// Re-enable stuff in core
					wp_suspend_cache_invalidation( false );
					wp_cache_flush();
					wp_defer_term_counting( false );
					wp_defer_comment_counting( false );

					if ( !empty( $quiz_questions ) ) {
						//update_post_meta( $quiz_post_id, 'ld_quiz_questions', $quiz_questions );
						$ld_qq = new LDLMS_Quiz_Questions($quiz_post_id);
						$ld_qq->set_questions($quiz_questions);
					}

					$quiz_edit_link = add_query_arg( array(
						'post'   => $quiz_post_id,
						'action' => 'edit'
					), admin_url( 'post.php' ) );

					//Attach existing question to selected quizzes indexes
					if(is_array($questions_form_data)) {
						foreach ( $questions_form_data as $question_id => $question_quizzes ) {
							if ( in_array( $current, $question_quizzes ) ) {
								self::linkQuestionToQuiz( $question_id, $quiz_post_id );
							}
						}
					}

					if($current+1 <= $total) {
						$next_sheet_index = $current+1;
					} else {
						$next_sheet_index = -1;
					}

					$quiz_view_link = get_permalink($quiz_post_id);

					if(!empty($quiz_post_id)) {
						$message = sprintf( __( '"<strong>%s</strong>" imported successfully <span><a href="%s">%s</a> <a href="%s">%s</a></span>', 'wn-ld-quiz-import-export' ), get_the_title( $quiz_post_id ), $quiz_edit_link, __('Edit'), $quiz_view_link, __('View') );
					} else {
						$message = sprintf( __( '<span class="ld_qie_import_log_error">Invalid %s sheet "<strong>%s</strong></span>"', 'wn-ld-quiz-import-export' ), learndash_get_custom_label('quiz'), $worksheet);
					}

					$response = array(
						'status' => 'success',
						'data' => array(
							'next_sheet_index' => $next_sheet_index,
							'total' => $total,
							'processed' => $current+1,
							'quiz_post_id' => $quiz_post_id,
							'message' => $message,
						)
					);
					restore_error_handler();
					$this->send_json($response);
				}
			}

			restore_error_handler();
			$this->send_json($response);
		}

		private function update_quiz_settings( $quiz_id ) {

			
		}

		private function manual_php_config() {
			//return;
			//Increase memory limit and execution time to avoid timeouts
			ini_set('memory_limit ', '128M');
			ini_set('max_execution_time', 15 * MINUTE_IN_SECONDS);
			ini_set('max_input_time', 15 * MINUTE_IN_SECONDS);
			//ini_set('max_input_vars', 100 * 1000);
		}

		public function get_quizzes_list_cb() {

			if ( ! wp_doing_ajax() ) {
				return;
			}

			if( !check_ajax_referer('ld-qie-ajax-nonce', 'ld_qie_ajax_nonce')) {
				return;
			}

			delete_option($this->file_path_transient_key);

			$response = $this->do_upload();

			if ( $response['status'] == 'success' ) {

				if ( ! empty( $this->worksheets ) ) {

					$sheet_quizzes = [];
					$quiz_titles = [];

					//Increase memory limit and execution time to avoid timeouts
					ini_set('memory_limit ', '128M');
					ini_set('max_execution_time', 15 * MINUTE_IN_SECONDS);
					ini_set('max_input_time', 15 * MINUTE_IN_SECONDS);
					ini_set('max_input_vars', 100 * 1000);

					foreach ( $this->worksheets as $worksheet_index => $worksheet ) {

						/** set the current active worksheet by name **/
						$this->phpExcel->setActiveSheetIndexByName( $worksheet );
						$activeSheet = $this->phpExcel->getActiveSheet();
						$first_cell_value = $activeSheet->getCell('A1')->getValue();
						$first_cell_value = WN_LD_Quiz_Import_Export_Helper::string_replace_spaces_to_underscores(sanitize_text_field($first_cell_value));

						if( $first_cell_value !== 'quiz_title' ) continue; //Skip sheet if first column value does not match

						$highestColumnName  = $activeSheet->getHighestDataColumn(); //Get last column

						$column_names_data = $activeSheet->rangeToArray("A1:{$highestColumnName}1",
							NULL,        	// Value that should be returned for empty cells
							false,        	// Should formulas be calculated (the equivalent of getCalculatedValue() for each cell)
							false,        	// Should values be formatted (the equivalent of getFormattedValue() for each cell)
							TRUE         	// Should the array be indexed by cell row and cell column
						);

						$column_names = array_map(function($column_name) {
							return WN_LD_Quiz_Import_Export_Helper::string_replace_spaces_to_underscores($column_name);
						}, array_values($column_names_data[1]));

						$column_values_data = $activeSheet->rangeToArray("A2:{$highestColumnName}2", NULL, false, false, TRUE);

						$column_values = array_values($column_values_data[2]);

						$row_data = array_combine($column_names, $column_values);

						$columns_to_retrieve = [
							'quiz_title',
							'certificate_awarded_for',
							'passing_percentage',
							'course',
							'lesson_or_topic',
							'certificate'
						];

						foreach ($row_data as $column_name => $column_value) {
							if(!in_array($column_name, $columns_to_retrieve)) {
								unset($row_data[$column_name]);
							}
						}

						if( isset( $row_data['course'] ) && !empty( $row_data['course'] ) ) {
							$course_post = get_post( $row_data['course'] );
							if( is_null( $course_post ) ) {
								$course_id = get_transient( 'ld_cie_course_' . $row_data['course'] );
								if( isset( $course_id ) && !empty( $course_id ) ) {
									$row_data['course'] = intval( $course_id );
								}
							}
						}

						if( isset( $row_data['lesson_or_topic'] ) && !empty( $row_data['lesson_or_topic'] ) ) {
							$lesson_or_topic_post = get_post( $row_data['lesson_or_topic'] );
							if( is_null( $lesson_or_topic_post ) ) {
								$lesson_id = get_transient( 'ld_cie_lesson_' . $row_data['lesson_or_topic'] );
								if( isset( $lesson_id ) && !empty( $lesson_id ) ) {
									$row_data['lesson_or_topic'] = intval( $lesson_id );
								}

								$topic_id = get_transient( 'ld_cie_topic_' . $row_data['lesson_or_topic'] );
								if( isset( $topic_id ) && !empty( $topic_id ) ) {
									$row_data['lesson_or_topic'] = intval( $topic_id );
								}
							}
						}

						$sheet_quizzes[ $worksheet ] = $row_data;

						$quiz_titles[] = ['index' => $worksheet_index, 'title' => $row_data['quiz_title']];
					}

					$setting_quiz_meta = [];

					$setting_quiz_post_id = absint($this->settings['setting_quiz_post_id']);

					if(!empty($setting_quiz_post_id)) {

						$set_pro_quiz_id   = learndash_get_setting( $setting_quiz_post_id, 'quiz_pro' );

						if ( ! empty( $set_pro_quiz_id ) ) {

							$set_meta = get_post_meta( $setting_quiz_post_id, '_sfwd-quiz', true );
							if ( is_array( $set_meta ) && ! empty( $set_meta ) ) {
								$setting_quiz_meta = $set_meta;
							}
						}
					}

					ob_start();
					include_once plugin_dir_path(WN_LD_QUIZ_IMPORT_EXPORT_FILE) . 'admin/partials/imported_quizzes_table.php';
					$html = ob_get_clean();

					$response = array(
						'status'=> 'success',
						'data' => array(
							'html' => $html,
							'quiz_titles' => $quiz_titles
						)
					);


				}
			}

			$this->send_json( $response );
		}

		private function linkQuestionToQuiz($question_post_id, $quiz_post_id) {

			$question_pro_id = get_post_meta( $question_post_id, 'question_pro_id', true );

			$ld_qq = new LDLMS_Quiz_Questions($quiz_post_id);

			if(!$ld_qq) {
				return;
			}

			$questions = get_post_meta( $quiz_post_id, 'ld_quiz_questions', true );

			if( is_array($questions) ) {
				$questions[$question_post_id] = $question_pro_id;
			} else {
				$questions = $ld_qq->get_questions('post_ids');
			}

			$ld_qq->set_questions($questions);
		}

		public function get_question_details_cb() {

			if ( ! wp_doing_ajax() ) {
				return;
			}

			if( !check_ajax_referer('ld-qie-ajax-nonce', 'ld_qie_ajax_nonce')) {
				return;
			}

			$response = array(
				'status'  => 'error',
				'message' => __( 'An error occurred while processing your request', 'wn-ld-quiz-import-export' )
			);

			if ( ! empty( $_POST ) && isset( $_POST['question_id'] ) ) {
				$question_post_id = absint( $_POST['question_id'] );
				$question_pro_id  = get_post_meta( $question_post_id, 'question_pro_id', true );

				$data             = leandash_get_question_pro_fields( $question_pro_id, array(
					'title',
					'category_name',
					'points',
					'question',
					'answer_type',
					'category_id'
				) );
				$data['question'] = wp_trim_words( $data['question'], 30, '...' );

				$data = array_merge(['id' => $question_post_id], $data);

				$response = array(
					'status' => 'success',
					'data'   => $data
				);
			}

			$this->send_json( $response );
		}

		private function send_json( $response ) {
			if ( $response['status'] == 'error' ) {
				wp_send_json( $response, 400 );
			}

			wp_send_json( $response );
			exit;
		}

		public function search_questions_cb() {

			if ( ! wp_doing_ajax() ) {
				return;
			}

			if( !check_ajax_referer('ld-qie-ajax-nonce', 'ld_qie_ajax_nonce')) {
				return;
			}

			$per_page_count = 10;

			$page = isset( $_GET['page'] ) ? absint( $_GET['page'] ) : 1;

			$args = array(
				'posts_per_page' => $per_page_count,
				'post_type'      => learndash_get_post_type_slug( 'question' ),
				'post_status'    => 'publish',
				'paged'          => $page,
			);

			if ( isset( $_GET['search'] ) ) {
				$args['s'] = sanitize_text_field( $_GET['search'] );
			}

			if ( isset( $_GET['existing_question_ids'] ) ) {
				$args['post__not_in'] = is_array( $_GET['existing_question_ids'] ) && ! empty( $_GET['existing_question_ids'] ) ? $_GET['existing_question_ids'] : array();
			}

			//$quizzes_options = learndash_get_option( learndash_get_post_type_slug('quiz') );
			$is_shared_quiz_questions = LearnDash_Settings_Section::get_section_setting( 'LearnDash_Settings_Quizzes_Builder', 'shared_questions' ) === 'yes';

			if ( ! $is_shared_quiz_questions ) {
				$args['meta_query'] = array(
					'relation' => 'OR',
					array(
						'key'     => 'quiz_id',
						'value'   => '',
						'compare' => 'NOT EXISTS'
					),
					array(
						'key'     => 'quiz_id',
						'value'   => '',
						'compare' => '='
					),
					array(
						'key'     => 'quiz_id',
						'value'   => '0',
						'compare' => '='
					)

				);
			}

			$question_query = new WP_Query( $args ); //Used inside template below
			$total_count    = $question_query->found_posts;
			$data           = array();

			foreach ( $question_query->posts as $post ) {
				$post_data = array(
					'id'   => $post->ID,
					'text' => "{$post->ID} - {$post->post_title}"
				);

				$data[] = $post_data;
			}

			$response = array( 'status' => 'success', 'data' => $data, 'count_filtered' => $total_count );

			$this->send_json( $response );
		}

		public function search_ld_post_type_cb() {

			if ( ! wp_doing_ajax() ) {
				return;
			}

			if( !check_ajax_referer('ld-qie-ajax-nonce', 'ld_qie_ajax_nonce')) {
				return;
			}

			$dropdown_type = isset( $_GET['dropdown_type'] ) ? sanitize_text_field($_GET['dropdown_type']) : null;

			$ld_post_types = [];

			if($dropdown_type==='courses') {
				$ld_post_types = learndash_get_post_type_slug( 'course' );
			} else if($dropdown_type==='lessons_topics') {
				$ld_post_types[] = learndash_get_post_type_slug( 'lesson' );
				$ld_post_types[] = learndash_get_post_type_slug( 'topic' );
			} else if($dropdown_type==='certificates') {
				$ld_post_types[] = learndash_get_post_type_slug( 'certificate' );
			}

			if( empty($ld_post_types) ) {
				return;
			}

			$per_page_count = 5;

			$page = isset( $_GET['page'] ) ? absint( $_GET['page'] ) : 1;

			$args = array(
				'posts_per_page' => $per_page_count,
				'post_type'      => $ld_post_types,
				'post_status'    => ['publish','pending','draft'],
				'paged'          => $page,
			);

			if ( isset( $_GET['search'] ) ) {
				$args['s'] = sanitize_text_field( $_GET['search'] );
			}

			/*if ( isset( $_GET['existing_question_ids'] ) ) {
				$args['post__not_in'] = is_array( $_GET["existing_question_ids"] ) && ! empty( $_GET['existing_question_ids'] ) ? $_GET['existing_question_ids'] : array();
			}*/

			//$quizzes_options = learndash_get_option( learndash_get_post_type_slug('quiz') );
			/*$is_shared_quiz_questions = LearnDash_Settings_Section::get_section_setting( 'LearnDash_Settings_Quizzes_Builder', 'shared_questions' ) === 'yes';

			if ( ! $is_shared_quiz_questions ) {
				$args['meta_query'] = array(
					'relation' => 'OR',
					array(
						'key'     => 'quiz_id',
						'value'   => '',
						'compare' => 'NOT EXISTS'
					),
					array(
						'key'     => 'quiz_id',
						'value'   => '',
						'compare' => '='
					),
					array(
						'key'     => 'quiz_id',
						'value'   => '0',
						'compare' => '='
					)

				);
			}*/

			$question_query = new WP_Query( $args ); //Used inside template below
			$total_count    = $question_query->found_posts;
			$data           = array();

			foreach ( $question_query->posts as $post ) {
				$post_data = array(
					'id'   => $post->ID,
					'text' => "{$post->ID} - {$post->post_title}"
				);

				$data[] = $post_data;
			}

			$response = array( 'status' => 'success', 'data' => $data, 'count_filtered' => $total_count );

			$this->send_json( $response );
		}

		public function search_quizzes_cb() {

			if ( ! wp_doing_ajax() ) {
				return;
			}

			if( !check_ajax_referer('ld-qie-ajax-nonce', 'ld_qie_ajax_nonce')) {
				return;
			}

			$per_page_count = 10;

			$page = isset( $_GET['page'] ) ? absint( $_GET['page'] ) : 1;

			$args = array(
				'posts_per_page' => $per_page_count,
				'post_type'      => learndash_get_post_type_slug( 'quiz' ),
				'post_status'    => ['publish','pending','draft'],
				'paged'          => $page,
			);

			if ( isset( $_GET['search'] ) ) {
				$args['s'] = sanitize_text_field( $_GET['search'] );
			}

			if ( isset( $_GET['existing_question_ids'] ) ) {
				$args['post__not_in'] = is_array( $_GET['existing_question_ids'] ) && ! empty( $_GET['existing_question_ids'] ) ? $_GET['existing_question_ids'] : array();
			}

			//$quizzes_options = learndash_get_option( learndash_get_post_type_slug('quiz') );
			/*$is_shared_quiz_questions = LearnDash_Settings_Section::get_section_setting( 'LearnDash_Settings_Quizzes_Builder', 'shared_questions' ) === 'yes';

			if ( ! $is_shared_quiz_questions ) {
				$args['meta_query'] = array(
					'relation' => 'OR',
					array(
						'key'     => 'quiz_id',
						'value'   => '',
						'compare' => 'NOT EXISTS'
					),
					array(
						'key'     => 'quiz_id',
						'value'   => '',
						'compare' => '='
					),
					array(
						'key'     => 'quiz_id',
						'value'   => '0',
						'compare' => '='
					)

				);
			}*/

			$question_query = new WP_Query( $args ); //Used inside template below
			$total_count    = $question_query->found_posts;
			$data           = array();

			foreach ( $question_query->posts as $post ) {
				$post_data = array(
					'id'   => $post->ID,
					'text' => "{$post->ID} - {$post->post_title}"
				);

				$data[] = $post_data;
			}

			$response = array( 'status' => 'success', 'data' => $data, 'count_filtered' => $total_count );

			$this->send_json( $response );
		}

		private function load_file_to_phpExcel( $file_path ) {
			$loaded = true;
			try {

				/** automatically detect the correct reader to load for this file type */
				$objReaderType = IOFactory::identify( $file_path );

				if( !in_array($objReaderType, ['Xls','Xlsx']) ) {
					$loaded = false;
				} else {

					$objReader = IOFactory::createReader( $objReaderType );

					$objReader->setReadDataOnly( true );

					/**  Load only the rows and columns that match our filter to PHPExcel  **/
					$this->phpExcel = $objReader->load( $file_path );
				}

			} catch (Exception $exception) {
				$loaded = false;
			}

			return $loaded;
		}

		public function do_upload() {

			$response = array(
				'status'  => 'error',
				'message' => __( 'An unknown error occurred while importing file.', 'wn-ld-quiz-import-export' )
			);

			//TODO implement wp_nonce check
			if ( wp_doing_ajax() && ! empty( $_POST ) && ! empty( $_FILES ) && is_user_logged_in() ) {

				$file_path = $this->upload_excel_file( $_FILES, [ 'xls','xlsx' ] );

				if ( ! empty( $file_path ) ) {

					if ( $file_path === 1 ) {
						$response['message'] = __( 'Only XLS or XLSX file types are supported.', 'wn-ld-quiz-import-export' );
					} else {

						set_transient($this->file_path_transient_key, $file_path, HOUR_IN_SECONDS); //Delete uploaded file

						update_option($this->file_path_transient_key, $file_path);

						$this->current_file_path = $file_path;

						$loaded = $this->load_all_worksheets();

						if($loaded) {

							//if ( count( $worksheets ) > 1 ) {
							//	$response['message'] = __( "Please upload a file with single worksheet, file(s) with multiple worksheets are not supported.", "ldqie" );
							//} else {

							$response = array(
								'status' => 'success'
							);
							//}
						}
					}
				}
			}

			return $response;
		}

		private function upload_excel_file( $files, $file_formats = [ 'xls', 'xlsx' ] ) {
			$file = isset( $files['ldqie_import_file'] ) ? $files['ldqie_import_file'] : null;
			if ( ! empty( $file ) ) {
				$ext = strtolower( pathinfo( sanitize_text_field( $file['name'] ), PATHINFO_EXTENSION ) );

				if ( in_array( $ext, $file_formats ) ) {
					global $current_user;
					$upload_dir   = wp_upload_dir();
					$user_dirname = $upload_dir['basedir'] . '/ld_aq_uploads/' . $current_user->ID;
					if ( ! file_exists( $user_dirname ) ) {
						wp_mkdir_p( $user_dirname );
					}

					$file_path = trailingslashit( $user_dirname ) . $file['name'];
					move_uploaded_file( $file["tmp_name"], $file_path );

					$this->file_loaded = true;

					return $file_path; //File upload successful
				}

				return 1; //Invalid file format
			}

			return 0; //Invalid file
		}

		/**
		 * Retrieve all answer_x keys from array
		 *
		 * @param $columns
		 *
		 * @return array
		 */
		private function get_answer_keys($array) {

			$answer_col_prefix = apply_filters('ld_qie_answer_col_prefix', 'answer_');

			return array_filter($array, function($key) {
				return strpos($key, 'answer_') === 0;
			}, ARRAY_FILTER_USE_KEY);
		}

		private function save_advanced_quizzes_question_meta($question_post_id, $columns) {

			//Skip if not advanced quizzes question type
			if( !in_array($columns['question'], [
				'laq_jumbled_sentence',
				'laq_swipe_questions',
				'laq_calculated_formula',
				'laq_jumble_dnd_sentence',
				'laq_audio_questions'
			]) ) {
				return;
			}

			$question_data = $columns;

			if( $question_data['question'] === 'laq_jumbled_sentence' ) {
				$sub_question_text = $question_data['sub_question_text'];
				$blank_option = $question_data['question_blank_option'];
				$randomize_option = $question_data['question_randomize_option'];
				// $correct_answers = array_filter(array_map('trim', explode('|', $question_data['answer'])), fn($v) => $v !== '' || $v === '0');
				$variables       = $this->get_shortcodes( $sub_question_text );
				/* if ( count( $correct_answers ) === count( $variables ) ) { */
				$options = [];

				foreach ( $variables as $index => $variable ) {
					$number = $index + 1;

					if ( isset( $question_data["answer_{$number}"] ) && ! empty( $question_data["answer_{$number}"] ) ) {
						$raw_answers = explode( '|', $question_data["answer_{$number}"] );
						$option      = [];

						$option['code']               = $variable;
						$option['vals']               = json_decode( $raw_answers[0] );
						$option['points']             = isset( $question_data["point_{$number}"] ) ? absint( $question_data["point_{$number}"] ) : 0;
						$option['correct']            = $question_data['answer'];
						$option['pts_for_each_usage'] = $raw_answers[1];

						$options[] = $option;
					}
				}

				update_post_meta( $question_post_id, '_ld_advanced_question_random_option_value', $randomize_option );
				update_post_meta( $question_post_id, '_ld_advanced_question_blank_option_value', $blank_option );
				update_post_meta( $question_post_id, '_ld_advanced_jumbled_question', $sub_question_text );
				update_post_meta( $question_post_id, '_ld_advanced_jumbled_options', $options );
				/*  }  */
			} else if( $question_data['question'] === 'laq_swipe_questions' ) {

				$answers = $this->get_answer_keys($question_data);

				$correct_answers = array_filter(array_map('absint',explode('|', $question_data['answer'])));
				
				if(count($answers) > 0) {
					$swipe_options = [];
					foreach( array_values($answers) as $answer_index => $answer_value  ) {
						$i = $answer_index + 1;
						$swipe_option_content = !empty(@$question_data["answer_{$i}"]) ? $question_data["answer_{$i}"] : '';
						$swipe_option_points = !empty(@$question_data["point_{$i}"]) ? absint($question_data["point_{$i}"]) : 0;

						if( !empty($swipe_option_content) ) {
							$option              = [];
							$option['question']  = $swipe_option_content;
							$option['points']    = $swipe_option_points;
							$option['correct']   = in_array( $i, $correct_answers ) ? 1 : 0;
							$option['attempted'] = null;

							$swipe_options[] = $option;
						}
					}

					update_post_meta($question_post_id,'_ld_advanced_swipe_questions', $swipe_options);
				}
			} else if( $question_data['question'] === 'laq_calculated_formula' ) {
				$sub_question_text = $question_data['sub_question_text'];
				$variables = $this->get_shortcodes( $sub_question_text );
				$answers = $this->get_answer_keys($question_data);

				if(count($answers) > 0) {
					$options = [];
					foreach( array_values($answers) as $answer_index => $answer_value  ) {
						$i = $answer_index + 1;
						$option_content = !empty($answer_value) ? trim($answer_value) : '';
						$option_points  = !empty( @$question_data["point_{$i}"] ) ? absint( $question_data["point_{$i}"] ) : 0;

						if( !empty($option_content) ) {
							$answer_data = explode('|', $option_content);
							$formula = $answer_data[0];
							unset($answer_data[0]);

							if(count($variables) === count($answer_data)) {
								$option             = [];
								$option['question'] = $sub_question_text;
								$option['formula']  = $formula;
								$option['text']     = $variables;
								$option['max']      = $option['min'] = [];
								foreach ( $variables as $vindex => $variable ) {
									list($min,$max) = explode(',', $answer_data[$vindex+1]);
									$option['min'][] = $min;
									$option['max'][] = $max;
								}
								$option['points'] = $option_points;
								$options[]        = $option;
							}
						}
					}

					update_post_meta($question_post_id, '_ld_advanced_calculated_formula_questions', $options);
				}
			} else if( $question_data['question'] === 'laq_jumble_dnd_sentence' ) {
				$sub_question_text = $question_data['sub_question_text'];
				$correct_answers = array_filter( array_map( 'trim', explode( '|', $question_data['answer'] ) ) );
				$variables       = $this->get_shortcodes( $sub_question_text );
				$options = [];

				for ($i = 1; isset($question_data["answer_$i"]); $i++) {

					if ( isset( $question_data["answer_{$i}"] ) && ! empty( $question_data["answer_{$i}"] ) ) {
						$raw_answers = explode( '|', $question_data["answer_{$i}"] );
						$option      = [];

						$option['code']               = json_decode( $raw_answers[0] );
						$option['vals']               = json_decode( $raw_answers[1] );
						$option['points']             = isset( $question_data["point_{$i}"] ) ? absint( $question_data["point_{$i}"] ) : 0;
						$option['correct']            = $correct_answers[ $index ];

						$option['pts_for_each_usage'] = $raw_answers[2];

						$options[] = $option;
					}
				}

				update_post_meta( $question_post_id, '_ld_advanced_dnd_jumbled_question', $sub_question_text );
				update_post_meta( $question_post_id, '_ld_advanced_dnd_jumbled_options', $options );
			
			} else if( $question_data['question'] === 'laq_audio_questions' ) {
				$sub_question_text = $question_data['sub_question_text'];
				$correct_answers = array_filter( array_map( 'trim', explode( '|', $question_data['answer'] ) ) );
				$variables       = $this->get_shortcodes( $sub_question_text );
				$options = [];
				foreach ( $variables as $index => $variable ) {
					$number = $index + 1;

					if ( isset( $question_data["answer_{$number}"] ) && ! empty( $question_data["answer_{$number}"] ) ) {
						$raw_answers = explode( '|', $question_data["answer_{$number}"] );
						$option      = [];

						$option['code']               = $variable;
						$option['vals']               = json_decode( $raw_answers[0] );
						$option['points']             = isset( $question_data["point_{$number}"] ) ? absint( $question_data["point_{$number}"] ) : 0;
						$option['correct']            = $correct_answers[ $index ];
						$option['pts_for_each_usage'] = $raw_answers[1];

						$options[] = $option;
					}
				}

				update_post_meta( $question_post_id, '_ld_advanced_dnd_jumbled_question', $sub_question_text );
				update_post_meta( $question_post_id, '_ld_advanced_dnd_jumbled_options', $options );
			}
		}

		private function update_advanced_quizzes_question_settings($question_post_id, $columns) {

			//Skip if LD Advanced Quizzes not activated
			if( !class_exists('Learndash_Advanced_Quiz') ) {
				return;
			}

			$additional_question_settings = trim(@$columns['additional_question_settings']);

			if(!empty($additional_question_settings)) {

				$advanced_quiz_settings = (array) json_decode( $additional_question_settings );

				if ( is_array( $advanced_quiz_settings ) ) {

					$ld_aq_settings = WN_LD_Quiz_Import_Export_Helper::get_advanced_quizzes_settings(); //LD Advance Quizzes Add-on settings

					if ( isset( $advanced_quiz_settings['negative_points'] ) ) {

						if ( $ld_aq_settings['negative_points_enabled'] ) {
							$negative_points = absint( $advanced_quiz_settings['negative_points'] );
							update_post_meta( $question_post_id, '_ld_retake_negative_points', $negative_points );
						}
					}

					if ( isset( $advanced_quiz_settings['time_limit'] ) ) {

						if ( $ld_aq_settings['question_time_limit_enabled'] ) {
							$time_limit = absint( $advanced_quiz_settings['time_limit'] );
							update_post_meta( $question_post_id, '_ld_advanced_time_limit', $time_limit );
						}
					}
				}
			}
		}

		/**
		 * Find matches enclosed in square brackets i.e. [x]
		 *
		 * @param $string
		 *
		 * @return array
		 */
		private function get_shortcodes( $string ) {
			$shortcodes = [];

			preg_match_all("/\[([^\]]*)\]/", $string, $matches, PREG_SET_ORDER);

			if ( !empty( $matches ) ) {
				foreach ( $matches as $shortcode ) {
					$shortcodes[] = $shortcode[1];
				}
			}

			return $shortcodes;
		}

		/**
		 * Create quiz with title in WP Pro Quiz and LearnDash if not same title does exists already
		 *
		 * @param array $quiz_data
		 * @param int $setting_quiz_post_id Post ID of the quiz to import settings from.
		 * @param bool $unique Skip quiz creation if name already exists, and return the ID instead
		 *
		 * @return int Quiz Pro ID
		 */
		function saveQuiz( $quiz_data = array(), $setting_quiz_post_id = 0, $form_quiz_data = array(), $unique = true ) {

			$quiz_post_id 	= 0;
			

			if ( ! empty( $quiz_data ) ) {

				//Retrieve existing
				if ( isset( $quiz_data['_name'] ) && $unique ) {

					$post = $this->getLDPostByTitle( $quiz_data['_name'], 'quiz' );
					if ( $post ) {
						$quiz_post_id = $post->ID;
					}
				}

				$quiz_content 	= empty( @$quiz_data['_quiz_content'] ) ?  '' : $quiz_data['_quiz_content'];
				$quiz_material 	= empty( @$quiz_data['_quiz_material'] ) ? '' : $quiz_data['_quiz_material'];
				$quiz_category 	= empty( @$quiz_data['_quiz_category'] ) ? '' : $quiz_data['_quiz_category'];
				$quiz_tags 		= empty( @$quiz_data['_quiz_tags'] ) ? '' : $quiz_data['_quiz_tags'];

				$quiz_data = $this->set_quiz_settings_in_quiz_data_on_import( $quiz_data );

				// Create new
				if ( empty( $quiz_post_id ) ) {
					$this->log_message(sprintf('Creating %s "%s"', learndash_get_custom_label_lower('quiz'), $quiz_data['_name']));
					$quizMapper     = new WpProQuiz_Model_QuizMapper();
					$pro_quiz_model = new WpProQuiz_Model_Quiz();

					if ( ! empty( $setting_quiz_post_id ) ) {

						$set_pro_quiz_id = learndash_get_setting( $setting_quiz_post_id, 'quiz_pro' );

						if ( ! empty( $set_pro_quiz_id ) ) {
							$set_quiz = $quizMapper->fetch( $set_pro_quiz_id );
							$set_quiz->setId( 0 );
							$set_quiz->setPostId( 0 );
							$set_quiz->setName( $quiz_data['_name'] );
							$quiz_data = $set_quiz->get_object_as_array();
						}
					}

					$pro_quiz_model->set_array_to_object( $quiz_data );

					$quizMapper->save( $pro_quiz_model );

					$pro_quiz_id = $pro_quiz_model->getId();

					$user_id          = get_current_user_id();

					$post_status = 'pending';
					if($this->settings['publish_quiz_on_import']) {
						$post_status = 'publish';
					}

					$quiz_insert_data = array(
						'post_type'   => learndash_get_post_type_slug( 'quiz' ),
						'post_title'  => $pro_quiz_model->getName(),
						'post_content'  => $quiz_content,
						'post_status' => $post_status,
						'post_author' => $user_id,
					);

					$quiz_post_id = wp_insert_post( $quiz_insert_data );

					WN_LD_Quiz_Import_Export_Helper::ld_quiz_import_export_category_import( $quiz_category, $quiz_post_id, 'category' );
					WN_LD_Quiz_Import_Export_Helper::ld_quiz_import_export_category_import( $quiz_tags, $quiz_post_id, 'post_tag' );
					
					if ( ! empty( $quiz_post_id ) ) {
						learndash_update_setting( $quiz_post_id, 'quiz_pro', $pro_quiz_id );

						$this->elementor_support_importer($quiz_post_id, $quiz_data);
					}
					
					// Template quiz settings set then get its post it from settings array and pass to our function
					// which will overwrite our imported quiz settings
					$quiz_settings = get_option( 'ld_qie_settings' );
	            	$template_quiz_id = $quiz_settings['setting_quiz_post_id'];
	            	$template_quiz_id = is_numeric( $template_quiz_id ) ? intval( $template_quiz_id ) : 0;
					
					if( $template_quiz_id > 0 ) {
	            		$this->save_template_quiz_settings( $quiz_post_id, $pro_quiz_id, $quizMapper );
	            	}

					$meta = $this->get_quiz_meta($quiz_post_id, $form_quiz_data);

					if ( ! empty( $quiz_material ) ) {
						$meta["sfwd-quiz_quiz_materials_enabled"] = 'on';
						$meta["sfwd-quiz_quiz_materials"] = wp_kses_post($quiz_material);
					}
					
					$meta[learndash_get_post_type_slug('quiz') . '_quiz_pro'] = $pro_quiz_id; // Set new pro quiz id
					update_post_meta( $quiz_post_id, '_sfwd-quiz', $meta );

					$course_id = absint( @$meta["sfwd-quiz_course"] );
					$lesson_id = absint( @$meta["sfwd-quiz_lesson"] );

					$this->associate_quiz_course_lesson($quiz_post_id,$course_id,$lesson_id);
					$post_meta_import_keys = array( '_viewProfileStatistics' => 1, '_timeLimitCookie' => 0 );

					foreach ( $post_meta_import_keys as $post_meta_import_key => $post_meta_import_value ) {
						if ( ! empty( $setting_quiz_post_id ) ) {
							$post_meta_import_value = get_post_meta( $setting_quiz_post_id, $post_meta_import_key, true );
							update_post_meta( $quiz_post_id, $post_meta_import_key, $post_meta_import_value );
						} else {
							update_post_meta( $quiz_post_id, $post_meta_import_key, $post_meta_import_value );
						}
					}

				} else {
					$this->log_message(sprintf('Updating %s "%s"', learndash_get_custom_label_lower('quiz'), $quiz_data['_name']));
					$quizMapper     = new WpProQuiz_Model_QuizMapper();
					$pro_quiz_model = new WpProQuiz_Model_Quiz();
					if ( ! empty( $setting_quiz_post_id ) ) {

						$set_pro_quiz_id = learndash_get_setting( $setting_quiz_post_id, 'quiz_pro' );

						if ( ! empty( $set_pro_quiz_id ) ) {
							$set_quiz = $quizMapper->fetch( $set_pro_quiz_id );
							$set_quiz->setId( 0 );
							$set_quiz->setPostId( 0 );
							$set_quiz->setName( $quiz_data['_name'] );
							$quiz_data = $set_quiz->get_object_as_array();
						}
					}

					$pro_quiz_model->set_array_to_object( $quiz_data );

					$valid_quiz_pro = false;
					$quiz_pro_id    = learndash_get_setting( $quiz_post_id, 'quiz_pro' );
					$quiz_pro_id    = absint( $quiz_pro_id );

					$quiz_pro    = $quizMapper->fetch( $quiz_pro_id );
					if ( ( is_a( $quiz_pro, 'WpProQuiz_Model_Quiz' ) ) && ( $quiz_pro_id === $quiz_pro->getId() ) ) {
						$valid_quiz_pro = true;
						$pro_quiz_model->setId( $quiz_pro_id ); //Set existing pro quiz ID if exists
					}

					$quizMapper->save( $pro_quiz_model );

					$post_status = 'pending';
					if($this->settings['publish_quiz_on_import']) {
						$post_status = 'publish';
					}

					$quiz_update_data = array(
						'ID'		  => $quiz_post_id,
						'post_type'   => learndash_get_post_type_slug( 'quiz' ),
						'post_title'  => $pro_quiz_model->getName(),
						'post_content'  => $quiz_content,
						'post_status' => $post_status,
						'post_author' => get_current_user_id(),
					);

					$quiz_post_id = wp_update_post( $quiz_update_data );

					// Template quiz settings set then get its post it from settings array and pass to our function
					// which will overwrite our imported quiz settings
					$quiz_settings = get_option( 'ld_qie_settings' );
	            	$template_quiz_id = $quiz_settings['setting_quiz_post_id'];
	            	$template_quiz_id = is_numeric( $template_quiz_id ) ? intval( $template_quiz_id ) : 0;

	            	if( $template_quiz_id > 0 ) {
	            		$this->save_template_quiz_settings( $quiz_post_id, $quiz_pro_id, $quizMapper );
	            	}

					$meta = $this->get_quiz_meta($quiz_post_id, $form_quiz_data);
					
					if(!empty($quiz_material)) {
						$meta["sfwd-quiz_quiz_materials_enabled"] = 'on';
						$meta["sfwd-quiz_quiz_materials"] = wp_kses_post($quiz_material);
					}
					
					if( $valid_quiz_pro===true ) {
						$meta[ learndash_get_post_type_slug( 'quiz' ) . '_quiz_pro' ] = $quiz_pro_id; //Set existing pro quiz id
					}

					update_post_meta( $quiz_post_id, '_sfwd-quiz', $meta );

					$course_id = absint( @$meta["sfwd-quiz_course"] );
					$lesson_id = absint( @$meta["sfwd-quiz_lesson"] );

					$this->associate_quiz_course_lesson($quiz_post_id,$course_id,$lesson_id);
					
				}
			}

			// Use this id to update quiz settings
			update_post_meta( $quiz_post_id, 'imported_quiz_id', $quiz_post_id );

			return $quiz_post_id;
		}


		private function elementor_support_importer($quiz_post_id, $quiz_data = array()){
			
			$elementor_edit_mode 	 = empty( @$quiz_data['_elementor_edit_mode'] ) ?  '' : $quiz_data['_elementor_edit_mode'];

			if( $elementor_edit_mode === 'builder' ){
				$elementor_template_type = empty( @$quiz_data['_elementor_template_type'] ) ?  '' : $quiz_data['_elementor_template_type'];
				$elementor_version 		 = empty( @$quiz_data['_elementor_version'] ) ?  '' : $quiz_data['_elementor_version'];
				$elementor_pro_version 	 = empty( @$quiz_data['_elementor_pro_version'] ) ?  '' : $quiz_data['_elementor_pro_version'];
				$elementor_page_assets 	 = empty( @$quiz_data['_elementor_page_assets'] ) ?  '' : $quiz_data['_elementor_page_assets'];
				$elementor_data 		 = empty( @$quiz_data['_elementor_data'] ) ?  '' : $quiz_data['_elementor_data'];
				$elementor_css 			 = empty( @$quiz_data['_elementor_css'] ) ?  '' : $quiz_data['_elementor_css'];
				
				update_post_meta($quiz_post_id, '_elementor_edit_mode', $elementor_edit_mode);
				update_post_meta($quiz_post_id, '_elementor_template_type', $elementor_template_type);
				update_post_meta($quiz_post_id, '_elementor_version', $elementor_version);
				update_post_meta($quiz_post_id, '_elementor_pro_version', $elementor_pro_version);
				update_post_meta($quiz_post_id, '_elementor_data', $elementor_data);
				if (is_serialized($elementor_page_assets)) {
					update_post_meta($quiz_post_id, '_elementor_page_assets', unserialize($elementor_page_assets));
				} else {
					update_post_meta($quiz_post_id, '_elementor_page_assets', json_decode($elementor_page_assets, true));
				}
				update_post_meta($quiz_post_id, '_elementor_css', json_decode($elementor_css, true));
			}

		}


		private function getQuestionModel( $question_columns, $is_shared_question ) {

			$questionModel = null;

			$question_columns['title'] = sanitize_text_field( $question_columns['title'] );

			$char_limit  = 100;
			if( strlen($question_columns['title']) > $char_limit ) { //trim down question title if greater than 100 characters
				$question_columns['title'] = mb_strimwidth($question_columns['title'], 0, $char_limit, '...');
			}

			if( !isset($question_columns['id']) ) {
				$question_columns['id'] = 0;
			} else {
				$question_columns['id'] = absint($question_columns['id']);
			}

			if( !isset($question_columns['quiz_id']) ) {
				$question_columns['quiz_id'] = 0;
			} else {
				$question_columns['quiz_id'] = absint($question_columns['quiz_id']);
			}

			if( !empty($question_columns['title']) ) {

				$questionModel = new WpProQuiz_Model_Question();

				$object_vars = array(
					'_id'                             => $question_columns['id'],
					'_quizId'                         => $question_columns['quiz_id'],
					'_title'                          => $question_columns['title'],
					'_question'                       => $this->sanitize_text( $question_columns['question_text'] ),
					'_correctMsg'                     => $question_columns['message_with_correct_answer'],
					'_incorrectMsg'                   => $question_columns['message_with_incorrect_answer'],
					'_correctSameText'                => sanitize_text_field( $question_columns['message_same_text'] )  == 'yes', //Use different message for incorrect answer
					'_tipEnabled'                     => !empty( $question_columns['hint'] ),
					'_tipMsg'                         => $this->sanitize_text( $question_columns['hint'] ),
					'_points'                         => (float)( $question_columns['total_point'] ),
					'_answerType'                     => $question_columns['question'],
					'_answerData'                     => $this->getAnswerData( $question_columns ),
					'_answerPointsActivated'          	=> sanitize_text_field( $question_columns['different_points_for_each_answer'] ) == 'yes',
					'_categoryName'                   	=> sanitize_text_field( $question_columns['category'] ),
					'_showPointsInBox'				  	=> sanitize_text_field( $question_columns['show_points_in_box'] ) == 'yes',	
					'_answerPointsDiffModusActivated'	=> sanitize_text_field( $question_columns['answer_points_diff_modus_activated'] ) == 'yes',	
				);

				$questionModel->set_array_to_object( $object_vars );
			}

			return $questionModel;
		}

		private function getAnswersArray( $question_columns ) {
			$answers = [];
			$c       = 1;
			for ( $i = 0; $i < count( $question_columns ); $i ++ ) {

				$answers['answer_type'] = $question_columns['question'];
				if ( isset( $question_columns['answer'] ) ) {
					$answers['answer'] = $question_columns['answer'];
				}

				$answer_key = "answer_{$c}";
				if ( !isset( $question_columns[ $answer_key ] ) ) {
					$answer_key = str_replace('_', '', $answer_key);
				}

				$points_key = "point_{$c}";
				if ( !isset( $question_columns[ $points_key ] ) ) {
					$points_key = str_replace('_', '', $points_key);
				}

				if ( isset( $question_columns[ $answer_key ] ) ) {
					$answer_option = array(
						'option' => $this->boolToString($question_columns[ $answer_key ]),
					);

					$answer_option['points'] = null;

					if( isset($question_columns[ $points_key ]) ) {
						$answer_option['points'] = $question_columns[ $points_key ];
					}

					$answers['options'][ $c ] = $answer_option;

					$c++;
				}
			}

			return $answers;
		}

		private function getAnswerData( $question_columns ) {

			/**
			 * [_answer:protected] => <strong>A</strong>
			 * [_html:protected] => 1
			 * [_points:protected] => 1
			 * [_correct:protected] =>
			 * [_sortString:protected] => <strong>A</strong>
			 * [_sortStringHtml:protected] =>
			 * [_graded:protected] =>
			 * [_gradingProgression:protected] => not-graded-none
			 * [_gradedType:protected] =>
			 * [_mapper:protected] =>
			 */
			$answers 	= $this->getAnswersArray( $question_columns );
			$points 	= (float)( $question_columns['total_point'] );
			$methodName = "parse_answer_{$answers['answer_type']}";

			if(method_exists($this, $methodName)) {
				if( 'parse_answer_essay' === $methodName ) {
					$answer_datas = $this->{$methodName}( $answers, $points );
				} else {
					$answer_datas = $this->{$methodName}( $answers );
				}
				
				foreach ($answer_datas as &$answer_data) {
					$answer_data['_answer'] = $this->sanitize_answer($answer_data['_answer'], $answers['answer_type']);
					$answer_data['_html'] = $this->has_html( $answer_data['_answer'] );
					if( $answers['answer_type'] === 'matrix_sort_answer' ) {
						$answer_data['_sortString'] = $this->sanitize_answer($answer_data['_sortString'], $answers['answer_type']);
						$answer_data['_sortStringHtml'] = ( $answer_data['_sortString'] != strip_tags( $answer_data['_sortString'] ) ); //return true if string has HTML, false otherwise
					}
				}

				return $answer_datas;
			}

			return [];
		}

		private function has_html($content) {

			$has_html = ( $content != strip_tags( $content ) ); //return true if content has HTML, false otherwise

			if($has_html === false) {

				preg_match_all(
					'/' . get_shortcode_regex() . '/',
					$content,
					$matches,
					PREG_SET_ORDER
				);

				$has_html = !empty($matches); //Short-codes exists
			}

			return $has_html;
		}

		private function sanitize_text($text) {
			// Check if the text contains MathML tags
			if (strpos($text, '<math') !== false) {
				return $this->convert_legacy_image_tag_to_html_tag($text);
			}

			$text = $this->convert_legacy_image_tag_to_html_tag( wp_kses_post( $text ));
			return $text;
		}

		private function sanitize_answer($text, $answer_type='') {
			// Check if the text contains MathML tags
			if (strpos($text, '<math') !== false) {
				return $this->convert_legacy_image_tag_to_html_tag($text, $answer_type, 'answer');
			}
			
			$text = $this->convert_legacy_image_tag_to_html_tag( wp_kses_post( $text ), $answer_type, 'answer' );

			return $text;
		}

		/**
		 * Converts bool values to their equivalent strings
		 *
		 * @param $string
		 *
		 * @return string
		 */
		private function boolToString($string){
			if( is_bool($string) ) {
				return ucwords(var_export($string, true));
			}

			return $string;
		}

		protected function parse_answer_single( $answer ) {
			$answers_data = [];
			foreach ( $answer['options'] as $option_key => $option ) {
				$answer_data             = [];
				$answer_data['_answer']  = $option['option'];
				$answer_data['_html']    = ( $answer_data['_answer'] != strip_tags( $answer_data['_answer'] ) ); //return true if string has HTML, false otherwise;
				$answer_data['_points']  = (float)( $option['points'] );
				$answer_data['_correct'] = !empty($answer['answer']) && absint( $option_key ) === absint( $answer['answer'] );
				$answers_data[]          = $answer_data;
			}

			return $answers_data;
		}

		protected function parse_answer_multiple( $answer ) {
			$answers_data    = [];
			$correct_answers = !empty($answer['answer']) ? array_filter( explode( '|', $answer['answer'] ) ) : array();
			foreach ( $answer['options'] as $option_key => $option ) {
				$answer_data             = [];
				$answer_data['_answer']  = $option['option'];
				$answer_data['_html']    = ( $answer_data['_answer'] != strip_tags( $answer_data['_answer'] ) ); //return true if string has HTML, false otherwise;
				$answer_data['_points']  = (float)( $option['points'] );
				$answer_data['_correct'] = in_array( $option_key, $correct_answers );
				$answers_data[]          = $answer_data;
			}


			return $answers_data;
		}

		protected function parse_answer_free_answer( $answer ) {
			$answers_data = [];

			$answer_data             = [];
			$answer_data['_answer']  = !empty($answer['answer']) ? sanitize_textarea_field( $answer['answer'] ) : '';
			$answer_data['_html']    = false;
			$answer_data['_correct'] = true;
			$answers_data[]          = $answer_data;

			return $answers_data;
		}

		protected function parse_answer_sort_answer( $answer ) {
			$answers_data = [];

			foreach ( $answer['options'] as $option_key => $option ) {
				$answer_data             = [];
				$answer_data['_answer']  = $option['option'];
				$answer_data['_html']    = ( $answer_data['_answer'] != strip_tags( $answer_data['_answer'] ) ); //return true if string has HTML, false otherwise;
				$answer_data['_points']  = (float)( $option['points'] );
				$answer_data['_correct'] = true;
				$answers_data[]          = $answer_data;
			}

			return $answers_data;
		}

		public function matrix_parse_string( $string ) {
		  	// Use a regular expression to match everything inside the brackets
		  	preg_match( '/\{(.+?)\}\{(.+?)\}/', $string, $matches );
	
		  	// If the regular expression doesn't match, return null
		  	if ( count( $matches ) != 3 ) {
		  	  	return null;
		  	}

		  	// Set $option and $sortString based on the matched values
		  	$option 	= $matches[1];
		  	$sortString = $matches[2];

		  	// Return an array with the parsed values
		  	return array(
		  	  $option,
		  	  $sortString
		  	);
		}

		protected function parse_answer_matrix_sort_answer( $answer ) {
			$answers_data = [];

			foreach ( $answer['options'] as $option_key => $option ) {

				$options = $this->matrix_parse_string( $option['option'] );

				if ( count( $options ) === 2 ) {
					list( $option['option'], $option['sortString'] ) = $options;
				}

				$answer_data                    = [];
				$answer_data['_answer']         = $option['option'];
				$answer_data['_html']           = ( $answer_data['_answer'] != strip_tags( $answer_data['_answer'] ) ); //return true if string has HTML, false otherwise;
				$answer_data['_points']         = (float)( $option['points'] );
				$answer_data['_correct']        = true;
				$answer_data['_sortString']     = $option['sortString'];
				$answer_data['_sortStringHtml'] = ( $answer_data['_sortString'] != strip_tags( $answer_data['_sortString'] ) ); //return true if string has HTML, false otherwise
				$answers_data[]                 = $answer_data;
			}

			return $answers_data;
		}

		protected function parse_answer_cloze_answer( $answer ) {
			$answers_data = [];

			$answer_data             = [];
			$answer_data['_answer']  = !empty($answer['answer']) ? wp_kses_post( $answer['answer'] ) : '';
			$answer_data['_html']    = ( $answer_data['_answer'] != strip_tags( $answer_data['_answer'] ) ); //return true if string has HTML, false otherwise;
			$answer_data['_correct'] = true;
			$answers_data[]          = $answer_data;

			return $answers_data;
		}

		protected function parse_answer_assessment_answer( $answer ) {
			$answers_data = [];

			$answer_data             = [];
			$answer_data['_answer']  = !empty($answer['answer']) ? wp_kses_post( $answer['answer'] ) : '';
			$answer_data['_html']    = ( $answer_data['_answer'] != strip_tags( $answer_data['_answer'] ) ); //return true if string has HTML, false otherwise;
			$answer_data['_correct'] = true;
			$answers_data[]          = $answer_data;

			return $answers_data;
		}

		protected function parse_answer_essay( $answer, $points ) {
			$answers_data    = [];
			$grading_options = !empty($answer['answer']) ? array_filter( array_map( 'sanitize_text_field', explode( '|', $answer['answer'] ) ) ) : array();
			if ( count( $grading_options ) === 2 ) {
				list( $graded_type, $grade_progression ) = $grading_options;
			}
			$answer_data             = [];
			$answer_data['_answer']  = '';
			$answer_data['_html']    = '';
			$answer_data['_points']  = (float)( $points );
			$answer_data['_correct'] = true;
			$answer_data['_graded']  = true;
			if ( isset( $grade_progression ) ) {
				$answer_data['_gradingProgression'] = $grade_progression;
			}
			if ( isset( $graded_type ) ) {
				$answer_data['_gradedType'] = $graded_type;
			}
			$answers_data[] = $answer_data;

			return $answers_data;
		}

		protected function parse_answer_laq_jumbled_sentence($answer) {
			$answers_data = [];
			return $answers_data;
		}

		protected function parse_answer_laq_swipe_questions($answer) {
			$answers_data = [];
			return $answers_data;
		}

		protected function parse_answer_laq_calculated_formula($answer) {
			$answers_data = [];
			return $answers_data;
		}

		private function getLDPostByTitle($title, $ld_post_type_slug) {
			$post = WN_LD_Quiz_Import_Export_Helper::wn_ldqie_get_page_by_title( $title, 'OBJECT', learndash_get_post_type_slug( $ld_post_type_slug ) );

			if ( ! empty( $post ) && in_array($post->post_status, ['publish','pending','draft']) ) {
				return $post;
			}

			return false;
		}
		
		// Import post type categories from excel
		public function learndash_quiz_import_export_post_categories( $categories, $post_id, $taxonomy ) {
			WN_LD_Quiz_Import_Export_Helper::multi_question_category_parse_categories_field( $categories, $post_id, $taxonomy );
		}

		public function saveQuestion( $question, $quiz_post_id, $is_shared_question = false, $skip_update = false ) {
		    $question_post_id = $question_pro_id = 0;

		    if ( get_class( $question ) !== 'WpProQuiz_Model_Question' ) {
		        return [$question_post_id, $question_pro_id];
		    }

		    $quiz_pro_id = absint( learndash_get_setting($quiz_post_id, 'quiz_pro') );

		    $db_question_exists = $quiz_question_exists = $update_question = false;

		    if ( $skip_update === true ) { // when $skip_update is true then we should replace old question with new
		        // Retrieve existing
		        $post = $this->getLDPostByTitle( $question->getTitle(), 'question' );
		        if ( $post ) {
		            $question_post_id = $post->ID;
		            $question_pro_id  = get_post_meta( $question_post_id, 'question_pro_id', true );
		            $db_question_exists = true;
		        }

		        if ( $db_question_exists ) { // Question with the same title exists in DB
		            if ( ! $is_shared_question ) {
		                $update_question = false;
		                $question_pro_id = 0;
		            } else {
		                $duplicate_questions = $this->get_questions_by_title_within_quiz( $quiz_post_id, $question->getTitle() );
		                if ( ! empty( $duplicate_questions ) ) {
		                    $dq_question_post_ids = array_keys( $duplicate_questions );
		                    $question_post_id = $dq_question_post_ids[0];
		                    $question_pro_id  = $duplicate_questions[ $question_post_id ];
		                    $quiz_question_exists = true;
		                }
		                $update_question = $quiz_question_exists;
		            }
		        }
		    }

		    $questionMapper = new WpProQuiz_Model_QuestionMapper();
		    $question->setId($question_pro_id);
		    $question->setQuizId($quiz_pro_id);

		    $pro_category_id = $question->getCategoryId();
		    $pro_category_name = $question->getCategoryName();
		    if (!empty($pro_category_name)) {
		        $categoryMapper = new WpProQuiz_Model_CategoryMapper();
		        $category = $categoryMapper->fetchByName($pro_category_name);
		        $categoryId = $category->getCategoryId();
		        if (!empty($categoryId) && absint($pro_category_id) !== absint($categoryId)) {
		            $question->setCategoryId($category->getCategoryId());
		            $question->setCategoryName($category->getCategoryName());
		        } else {
		            $category->setCategoryName($question->getCategoryName());
		            $category = $categoryMapper->save($category);
		            $question->setCategoryId($category->getCategoryId());
		            $question->setCategoryName($category->getCategoryName());
		        }
		    }

		    $question = $questionMapper->save($question);
		    $question_pro_id = absint($question->getId());
		    $user_id = get_current_user_id();
		    $question_post_array = array(
		        'post_type' => learndash_get_post_type_slug('question'),
		        'post_title' => $question->getTitle(),
		        'post_content' => $question->getQuestion(),
		        'post_status' => 'publish',
		        'post_author' => $user_id,
		    );

		    $question_post_array = wp_slash($question_post_array);

		    if (!$update_question && !$db_question_exists) {
		        // Check if the question already exists in the current quiz
		        $existing_questions = $this->get_questions_by_title_within_quiz($quiz_post_id, $question->getTitle());
		        if (empty($existing_questions) || $skip_update === false) {
		            $question_post_id = wp_insert_post($question_post_array);
		            if (!is_wp_error($question_post_id)) {
		                $this->log_message(sprintf('%s "%s" inserted', learndash_get_custom_label('Question'), $question->getTitle()));
		            }
		        }
		    } else {
		        $question_post_array['ID'] = $question_post_id;
		        $question_post_id = wp_update_post($question_post_array);
		        if (!is_wp_error($question_post_id)) {
		            $this->log_message(sprintf('%s "%s" updated', learndash_get_custom_label('Question'), $question->getTitle()));
		        }
		    }

		    if (!empty($question_post_id)) {
		        $this->attach_question_to_quiz($question_post_id, $question, $quiz_post_id);
		    }

		    return [$question_post_id, $question_pro_id];
		}


		/**
		 * Find questions with title in given quiz
		 *
		 * @param $quiz_post_id
		 * @param $question_title
		 *
		 * @return array|bool
		 */
		protected function get_questions_by_title_within_quiz($quiz_post_id, $question_title) {

			if( empty($quiz_post_id) ) return false;

			$questions = array();

			if ( LearnDash_Settings_Section::get_section_setting( 'LearnDash_Settings_Quizzes_Builder', 'shared_questions' ) === 'yes' ) {
				$questions_query_meta_query = array(
					'relation' => 'OR',
					array(
						'key'     => 'quiz_id',
						'value'   => $quiz_post_id,
						'compare' => '=',
					),
					array(
						'key'     => 'ld_quiz_' . $quiz_post_id,
						'value'   => $quiz_post_id,
						'compare' => '=',
					),
				);
			} else {
				// We clear out the existing questions to force query fresh from question posts.
				//$this->questions['post_ids'] = array();
				$questions_query_meta_query  = array(
					array(
						'key'     => 'quiz_id',
						'value'   => $quiz_post_id,
						'compare' => '=',
					),
				);
			}

			$questions_query_args = array(
				'title' => $question_title,
				'post_type'      => learndash_get_post_type_slug( 'question' ),
				'posts_per_page' => -1,
				'post_status'    => 'publish',
				'fields'         => 'ids',
				'orderby'        => 'menu_order',
				'order'          => 'ASC',
				'meta_query'     => $questions_query_meta_query,
			);

			$questions_query = new WP_Query( $questions_query_args );

			if ( ( is_a( $questions_query, 'WP_Query' ) ) && ( property_exists( $questions_query, 'posts' ) ) && ( ! empty( $questions_query->posts ) ) ) {
				foreach ( $questions_query->posts as $question_post_id ) {
					if ( ! isset( $questions[ $question_post_id ] ) ) {
						$question_pro_id = get_post_meta( $question_post_id, 'question_pro_id', true );
						if ( ! empty( $question_pro_id ) ) {
							$questions[ $question_post_id ] = (int) $question_pro_id;
						}
					}
				}
			}

			return $questions;
		}

		protected function attach_question_to_quiz($question_post_id, $question_pro_id, $quiz_post_id) {

			if( get_post_type($question_post_id) !== learndash_get_post_type_slug('question') ) return; //Return if not question type post

			if ( is_a( $question_pro_id, 'WpProQuiz_Model_Question' ) ) {
				$question = $question_pro_id;
			} else {
				$question_pro_mapper = new WpProQuiz_Model_QuestionMapper();
				$question = $question_pro_mapper->fetch( absint( $question_pro_id ) );
			}

			//$quiz_pro_id = absint(learndash_get_setting($quiz_post_id, 'quiz_pro'));

			//Following code was working and tested.
			/*update_post_meta( $question_post_id, 'points', absint( $question->getPoints() ) );
			update_post_meta( $question_post_id, 'question_type', $question->getAnswerType() );
			update_post_meta( $question_post_id, 'question_pro_id', absint( $question->getId() ) );

			learndash_update_setting( $question_post_id, 'quiz', $quiz_post_id );
			add_post_meta( $question_post_id, 'ld_quiz_id', $quiz_post_id );*/

			//Used following LearnDash provided functions to update same details above, instead of doing it manually
			learndash_update_setting( $question_post_id, 'quiz', $quiz_post_id );
			learndash_proquiz_sync_question_fields( $question_post_id, $question );
			learndash_proquiz_sync_question_category( $question_post_id, $question );
			learndash_set_question_quizzes_dirty( $question_post_id );
		}

		private function load_all_worksheets(  ) {

			$loaded = false;

			if(empty( $this->phpExcel ) ) {
				$loaded = $this->load_file_to_phpExcel( $this->current_file_path );

				if($loaded) {
					/** get all sheet names from the file **/
					$this->worksheets = $this->phpExcel->getSheetNames( $this->current_file_path );
				}
			}

			return $loaded;
		}


		private function convert_legacy_image_tag_to_html_tag( $text, $answer_type='', $column = '' ) {

			preg_match_all("/\[[^\]]*\]/", $text, $matches);
			$found = $matches[0];
			$final_text = $text;
			if( count( $found ) > 0 ) {

				$allowed_imgs = [ 'png', 'jpg', 'jpeg', 'gif', 'tiff' ];
				foreach( $found as $str ) {

					$temp = str_replace("[", "", $str);
					$temp = str_replace("]", "", $temp);

					$may_be_url = substr( $temp, -5 );
					$is_img = false;
					$image = '';
					if( strpos( $temp, '.' ) !== false ) {
						$img_ext = explode( '.', $may_be_url );
						if( in_array( trim( @$img_ext[1] ), $allowed_imgs ) ) {
							$is_img = true;
							$image = '<img src="'.trim( $temp ).'" />'. ' ';

							if( 'assessment_answer' == $answer_type && 'answer' == $column ) {
								$image = '[<img src="'.trim( $temp ).'" />]'. ' ';
							}
						}
					}

					if( !$is_img ) {
						if( 'assessment_answer' == $answer_type ) {
							if( 'answer' == $column ) {
								if( 'latex' == $temp ) {
									$image = '[['.$temp.']';
								} elseif( '/latex' == $temp ) {
									$image = '['.$temp.']]';
								}
							} else {
								$image = '['.$temp.']';
							}
						} else {
							$image = '['.$temp.']';
						}
					}


					if( !empty( $image ) ) {
						$final_text = str_replace( $str, $image, $final_text );
					}


				}
			}

			return $final_text;
		}

		private function delete_question($question_pro_id, $question_post_id) {

			global $wpdb;

			$question_pro_table = LDLMS_DB::get_table_name('question');
			$question_pro_delete_query = "DELETE FROM {$question_pro_table} QP WHERE QP.id=%d";
			$wpdb->query($wpdb->prepare($question_pro_delete_query, $question_pro_id));

			$question_post_delete_query = "DELETE P,PM 
											FROM {$wpdb->posts} P 
											JOIN {$wpdb->postmeta} PM ON PM.post_id = P.id
											WHERE P.post_type = %s AND P.ID = %d";

			$wpdb->query( $wpdb->prepare( $question_post_delete_query, $question_post_id ) );
		}

		private function override_form_quiz_meta($form_quiz_data, &$meta) {
			if ( !empty($form_quiz_data) ) {
				$form_passing_percentage = absint(@$form_quiz_data['passing_percentage']);
				$form_course_id = absint(@$form_quiz_data['course']);
				$form_lesson_id = absint(@$form_quiz_data['lesson_topic']);
				$form_certificate_id = absint(@$form_quiz_data['certificate']);
				$form_certificate_percent = absint(@$form_quiz_data['certificate_awarded_for']);
				if($form_passing_percentage === 0 || !empty($form_passing_percentage)) {
					$meta["sfwd-quiz_passingpercentage"] = $form_passing_percentage;
				}

				if(!empty($form_course_id)) {
					$meta["sfwd-quiz_course"] = $form_course_id;
				}

				if( !empty($form_lesson_id) ) {

					$lesson_topic_course_id = absint( learndash_get_course_id($form_lesson_id, true) );

					if( $lesson_topic_course_id === $form_course_id ) { //Check if course ID is matched against lesson/topic course ID
						$meta["sfwd-quiz_lesson"] = $form_lesson_id;
					}
				}

				if(!empty($form_certificate_id)) {
					$meta["sfwd-quiz_certificate"] = $form_certificate_id;
				}

				if($form_certificate_percent === 0 || !empty($form_certificate_percent)) {
					$meta["sfwd-quiz_threshold"] = ($form_certificate_percent / 100);
				}

				
			}

		}

		private function get_quiz_meta($setting_quiz_post_id,$form_quiz_data) {

			//Copy existing quiz post meta to new quiz post, if setting quiz post and pro ID are available
			$meta = [];
			if ( !empty($setting_quiz_post_id) ) {

				$set_meta = get_post_meta( $setting_quiz_post_id, '_sfwd-quiz', true );
				if ( is_array( $set_meta ) && ! empty( $set_meta ) ) {
					$meta = $set_meta;
				}
			}

			$this->override_form_quiz_meta($form_quiz_data, $meta);

			return $meta;
		}

		private function associate_quiz_course_lesson($quiz_post_id,$course_id,$lesson_id) {

			if( !empty($course_id) || !empty($lesson_id) ) {

				// Builder section issue fix
				$course_section_key = get_post_meta( $course_id, 'course_sections', true );
				$course_section = ( $course_section_key !== null ) ? get_post_meta( $course_id, 'course_sections', true ) : [];

				//TODO: When shared steps is enabled, import form fields should have multiple selection of courses and lessons or topics
				if ( !empty($course_id) && LearnDash_Settings_Section::get_section_setting('LearnDash_Settings_Courses_Builder', 'shared_steps' ) == 'yes' ) {

					$ld_course_steps_object = LDLMS_Factory_Post::course_steps( $course_id );

					if( !is_null($ld_course_steps_object) ) { //$ld_course_steps_object can be null, if invalid course ID passed

						$ld_course_steps_object->load_steps();
						$steps = $ld_course_steps_object->get_steps();

						if( !empty($lesson_id) ) {

							$parent_step_id = learndash_course_get_single_parent_step($course_id,$lesson_id);

							$post_type = get_post_type($lesson_id);

							if( $post_type === learndash_get_post_type_slug('lesson') && empty($parent_step_id) ) {
								$steps['sfwd-lessons'][ $lesson_id ]['sfwd-quiz'][ $quiz_post_id ] = array();
							} else if( $post_type === learndash_get_post_type_slug('topic') && $parent_step_id !== 0) {
								$steps['sfwd-lessons'][ $parent_step_id ]['sfwd-topic'][ $lesson_id ]['sfwd-quiz'][ $quiz_post_id ] = array();
							}

						} else {
							$steps['sfwd-quiz'][ $quiz_post_id ] = array();
						}

						$ld_course_steps_object->set_steps( $steps );
						$this->log_message(sprintf('%s #%s mapped to course/lesson/topic', learndash_get_custom_label('Quiz'), $quiz_post_id));
					}

				} else {

					if ( !empty($course_id) ) {
						update_post_meta( $quiz_post_id, 'course_id', $course_id );
						$this->log_message(sprintf('%s "%s" mapped to %s "%s"', learndash_get_custom_label('Quiz'), get_the_title($quiz_post_id), learndash_get_custom_label_lower('course'), get_the_title($course_id)));
					}

					if ( !empty($lesson_id) ) {
						update_post_meta( $quiz_post_id, 'lesson_id', $lesson_id );
						$this->log_message(sprintf('%s "%s" mapped to %s "%s"', learndash_get_custom_label('Quiz'), get_the_title($quiz_post_id), learndash_get_custom_label_lower('lesson'), get_the_title($lesson_id)));
					}
				}
				update_post_meta( $course_id, 'course_sections', $course_section );
			}
		}

		public function pre_wp_unique_post_slug_cb( $override_slug, $slug, $post_id, $post_status, $post_type, $post_parent ) {
			if ( $post_type === learndash_get_post_type_slug( 'question' ) ) {
				$override_slug = $slug . '-' . uniqid( 'ld-qie-', true );
			}

			return $override_slug;
		}

		/**
		 * This function will set quiz settings coming from our plugin to imported quiz
		*/
		public function set_quiz_settings_in_quiz_data_on_import( $data ) {
			
			$options = get_option( '__ldqie_plugin_quiz_settings' );
				    
			$data['_autostart'] = ! empty( $options['ldqie_quiz_autostart'] ) ? true : false;
			$data['_startOnlyRegisteredUser'] 	 = ! empty( $options['ldqie_allowed_users'] ) ? true : false;
			$data['_forcingQuestionSolve'] 	 	 = ! empty( $options['ldqie_question_completion'] ) ? true : false;
			$data['_hideQuestionNumbering']	 	 = ! empty( $options['ldqie_question_number'] ) ? false : true;
			$data['_showReviewQuestion']		 = ! empty( $options['ldqie_question_overview'] ) ? true : false;

			$question_display = ! empty( $options['ldqie_question_display'] ) ? $options['ldqie_question_display'] : 'single';
		    
		    if( 'single' === rtrim( $question_display ) ){

		    	$single_feedback = isset( $options['ldqie_question_display_result'] ) ? $options['ldqie_question_display_result'] : 'each';
		    	if( 'each' === $single_feedback ) {
		    		$data['_quizModus'] = 2;
		    	} elseif( 'end' === $single_feedback ) {
		    		$data['_quizModus'] = 1;
		    	}

		    } elseif( 'multiple' === rtrim( $question_display ) ) {
				    	
		    	$data['_quizModus'] = 3;
		    	$data['_questionsPerPage'] = ! empty( $options['ldqie_question_per_page'] ) ? absint( $options['ldqie_question_per_page'] ) : 0;

		    } 

		    return $data;
		}

		public function save_quiz_post( $post_ID, $post, $update ) {

			update_post_meta( $post_ID, 'imported_quiz_id', '0' );
		}

        /**
         * Copies/saves template quiz settings to imported quiz.
         *
         * @param int $post_id The ID of the post associated with the quiz.
         * @param int $pro_quiz_id The ID of the ProQuiz object.
         * @param object $ProQuiz_Mapper The ProQuiz mapper object used to save/map the pro quiz data.
         * @return void
         */
        public function save_template_quiz_settings( $post_id, $pro_quiz_id, $ProQuiz_Mapper ) {

            $quiz_settings = get_option( 'ld_qie_settings' );
            $template_quiz_id = $quiz_settings['setting_quiz_post_id'];
            $template_quiz_id = is_numeric( $template_quiz_id ) ? intval( $template_quiz_id ) : 0;

            if ( $template_quiz_id > 0 ) {

                $template_quiz_post_meta = get_post_meta( $template_quiz_id, '_sfwd-quiz', true );
                $template_quiz_pro_id = get_post_meta( $template_quiz_id, 'quiz_pro_id', true );

                if ( ! empty( $template_quiz_post_meta ) ) {

                    // Replace quiz pro id with newly generated imported quiz pro id in template quiz post meta.
                    $template_quiz_post_meta[ 'sfwd-quiz_quiz_pro' ] = $pro_quiz_id;

                    // Copy template post meta to newly generated imported quiz post meta.
                    update_post_meta( $post_id, '_sfwd-quiz', $template_quiz_post_meta );

                    // Update settings in learndash_pro_quiz_master for imported quiz.
                    $template_quiz_data = $ProQuiz_Mapper->fetch( $template_quiz_pro_id )->get_object_as_array();
                    $remove_keys = [ '_id', '_name', '_quiz_post_id', '_quiz_content', '_quiz_category', '_quiz_tags', '_quiz_material' ];
                    foreach ( $remove_keys as $key ) {
                        unset( $template_quiz_data[ $key ] );
                    }
                    
                    if( isset( $template_quiz_post_meta['sfwd-quiz_showCategory'] ) ) {
                    	$template_quiz_data['_showCategory'] = 1;
                    } 

                    if( isset( $template_quiz_post_meta['sfwd-quiz_showCategoryScore'] ) &&
                    	$template_quiz_post_meta['sfwd-quiz_showCategoryScore'] === 'on' ) {
                    	$template_quiz_data['_showCategoryScore'] = 1;
                    }
                    
                    $imported_quiz_data = $ProQuiz_Mapper->fetch( $pro_quiz_id );
                    
                    $imported_quiz_data->set_array_to_object( $template_quiz_data );

                    // Set data from template quiz which are not saving properly.
                    $imported_quiz_data->setShowCategory( $template_quiz_data['_showCategory'] );
                    $imported_quiz_data->setShowCategoryScore( $template_quiz_data['_showCategoryScore'] );
                    $ProQuiz_Mapper->save( $imported_quiz_data );

                }
            }
        }
	}
}
