<?php
/**
 * Helper function class to be used globally
 *
 * @link       https://wooninjas.com/
 * @since      1.0.0
 *
 * @package    WN_LD_Quiz_Import_Export
 * @subpackage WN_LD_Quiz_Import_Export/includes
 * @since 1.0.0
 */
if( !class_exists('WN_LD_Quiz_Import_Export_Helper') ) {

	class WN_LD_Quiz_Import_Export_Helper {

		/**
		 * Debug Log
		 *
		 * @param $var
		 * @param bool $print
		 * @param bool $show_execute_at
		 */
		public static function debug_log($var, $print=true, $show_execute_at=false) {
			ob_start();

			if($show_execute_at) {
				$bt = debug_backtrace();
				$caller = array_shift($bt);
				$execute_at = $caller['file'] . ':' . $caller['line'] . "\n";
				echo $execute_at;
			}

			if( $print ) {
				if( is_object($var) || is_array($var) ) {
					echo print_r($var, true);
				} else {
					echo $var;
				}
			} else {
				var_dump($var);
			}

			error_log(ob_get_clean());
		}

		/**
		 * TODO: Implement global accessible method to retrieve add-on settings
		 *
		 * Retrieve add-on settings
		 *
		 * @return array
		 */
		public static function get_settings() {

			$default_settings = [
				'delete_old_questions' => false, //Not in use
				'discard_old_questions' => 'no',
				'required_capability' => 'manage_options',
				'publish_quiz_on_import' => false,
				'import_existing_questions' => false,
				'setting_quiz_post_id' => 0,
				'update_duplicate_questions' => false,
				'activity_log' => false,
				'debug_log' => false,
			];

			$saved_settings = get_option('ld_qie_settings', []);

			$settings = wp_parse_args($saved_settings, $default_settings);

			//Old settings compatibility
			if( is_bool($settings['discard_old_questions']) ) {

				if($settings['discard_old_questions']) {
					$settings['discard_old_questions'] = 'quiz';
				} else {
					$settings['discard_old_questions'] = 'no';
				}
			}

			return $settings;
		}

		/**
		 * Retrieve LearnDash Advanced Quizzes add-on settings
		 *
		 * @return array
		 */
		public static function get_advanced_quizzes_settings() {

			if( class_exists('LD_AQ_Helper') && method_exists('LD_AQ_Helper', 'get_settings') ) {
				$settings = LD_AQ_Helper::get_settings();
			} else {
				/**
				 * For compatibility sake of LD AQ version 1.9 or less
				 */
				$setting_keys = array(
					'ld_retake_negative_marking' => 'negative_points_enabled',
					'ld_advanced_question_time_limit' => 'question_time_limit_enabled',
					'ld_advanced_save_resume_quiz' => 'save_quiz_enabled',
					'ld_advanced_save_as_next' => 'save_as_next_enabled'
				);

				$settings = [];

				foreach ($setting_keys as $setting_option_key => $setting_key) {
					$settings[$setting_key] = strtolower( trim( get_option( $setting_option_key, 'no' ) ) ) === 'yes';
				}
			}

			return $settings;
		}

		public static function doing_cron() {

			// Bail if not doing WordPress cron (>4.8.0)
			if ( function_exists( 'wp_doing_cron' ) && wp_doing_cron() ) {
				return true;

				// Bail if not doing WordPress cron (<4.8.0)
			} elseif ( defined( 'DOING_CRON' ) && ( true === DOING_CRON ) ) {
				return true;
			}

			// Default to false
			return false;
		}

		public static function string_replace_spaces_to_underscores($string) {
			return strtolower( preg_replace( '/\s+/', '_', trim( $string ) ) ); //convert string spaces to underscores
		}

		/**
		 * Format term ids to names.
		 *
		 * @since 3.7.5
		 * @param  array  $term_ids Term IDs to format.
		 * @param  string $taxonomy Taxonomy name.
		 * @return string
		 */
		public static function multi_question_category_format_term_ids( $term_ids, $taxonomy ) {
			$term_ids = wp_parse_id_list( $term_ids );

			if ( ! count( $term_ids ) ) {
				return '';
			}

			$formatted_terms = array();

			if ( is_taxonomy_hierarchical( $taxonomy ) ) {
				foreach ( $term_ids as $term_id ) {
					$formatted_term = array();
					$ancestor_ids   = array_reverse( get_ancestors( $term_id, $taxonomy ) );

					foreach ( $ancestor_ids as $ancestor_id ) {
						$term = get_term( $ancestor_id, $taxonomy );
						if ( $term && ! is_wp_error( $term ) ) {
							$formatted_term[] = $term->name;
						}
					}

					$term = get_term( $term_id, $taxonomy );

					if ( $term && ! is_wp_error( $term ) ) {
						$formatted_term[] = $term->name;
					}

					$formatted_terms[] = implode( ' > ', $formatted_term );
				}
			} else {
				foreach ( $term_ids as $term_id ) {
					$term = get_term( $term_id, $taxonomy );

					if ( $term && ! is_wp_error( $term ) ) {
						$formatted_terms[] = $term->name;
					}
				}
			}

			return WN_LD_Quiz_Import_Export_Helper::multi_question_category_implode_values( $formatted_terms );
		}

		/**
		 * Parse a category field from a CSV.
		 * Categories are separated by commas and subcategories are "parent > subcategory".
		 *
		 * @param string $value Field value.
		 *
		 * @return array of arrays with "parent" and "name" keys.
		 */
		public static function multi_question_category_parse_categories_field( $value, $question_title, $question_post_id ) {
			if ( empty( $value ) ) {
				return array();
			}

			$row_terms  = WN_LD_Quiz_Import_Export_Helper::multi_question_category_explode_values( $value );
			$categories = array();

			foreach ( $row_terms as $row_term ) {
				$parent = null;
				$_terms = array_map( 'trim', explode( '>', $row_term ) );
				$total  = count( $_terms );

				foreach ( $_terms as $index => $_term ) {
					// Don't allow users without capabilities to create new categories.
					if ( ! current_user_can( 'manage_options' ) ) {
						break;
					}

					if ( $parent !== null ){
						$_term = sanitize_text_field($_term);
						$parent = intval($parent);
						global $wpdb;
						// Database query to check if the term already exists
						$existing_term_query = $wpdb->prepare(
							"SELECT term.term_id
							FROM {$wpdb->terms} AS term
							INNER JOIN {$wpdb->term_taxonomy} AS tax ON term.term_id = tax.term_id
							WHERE term.name = %s AND tax.taxonomy = %s AND tax.parent = %d",
							$_term,
							'multi_question_category',
							$parent
						);

						$existing_term_id = $wpdb->get_var($existing_term_query);

						if ($existing_term_id) {
							// Term already exists
							$term_id = $existing_term_id;
						} else {
							$term = wp_insert_term( $_term, 'multi_question_category', array( 'parent' => intval( $parent ) ) );
							if ( is_wp_error( $term ) ) {
								if ( $term->get_error_code() === 'term_exists' ) {
									// When term exists, error data should contain existing term id.
									$term_id = $term->get_error_data();
	
								} else {
									break; // We cannot continue on any other error.
								}
							} else {
								// New term.
								$term_id = $term['term_id'];
							}
						}
					} else {
						$term = wp_insert_term( $_term, 'multi_question_category', array( 'parent' => intval( $parent ) ) );
						if ( is_wp_error( $term ) ) {
							if ( $term->get_error_code() === 'term_exists' ) {
								// When term exists, error data should contain existing term id.
								$term_id = $term->get_error_data();

							} else {
								break; // We cannot continue on any other error.
							}
						} else {
							// New term.
							$term_id = $term['term_id'];
						}
					}

					// old code
					// $term = wp_insert_term( $_term, 'multi_question_category', array( 'parent' => intval( $parent ) ) );

					// if ( is_wp_error( $term ) ) {
					// 	if ( $term->get_error_code() === 'term_exists' ) {
					// 		
					// 		// When term exists, error data should contain existing term id.
					// 		$term_id = $term->get_error_data();

					// 	} else {
					// 		break; // We cannot continue on any other error.
					// 	}
					// } else {
					// 	// New term.
					// 	$term_id = $term['term_id'];
					
					// }

					// Only requires assign the last category.
					if ( ( 1 + $index ) === $total ) {
						$categories[] = $term_id;
					} else {
						// Store parent to be able to insert or query categories based in parent ID.
						$parent = $term_id;
						wp_set_post_terms( $question_post_id, array( $parent ), 'multi_question_category', true );
					}
					wp_set_post_terms( $question_post_id, $categories, 'multi_question_category', true );
				}
			}

			return $categories;
		}

		/**
		 * Categories/tags are separated by commas and subcategories are "parent > subcategory".
		 *
		 * @param string $value Field value.
		 *
		 * @return array of arrays with "parent" and "name" keys.
		 */
		public static function ld_quiz_import_export_category_import( $value, $post_id, $taxonomy ) {
			if ( empty( $value ) ) {
				return array();
			}

			$row_terms  = WN_LD_Quiz_Import_Export_Helper::multi_question_category_explode_values( $value );
			$categories = array();

			foreach ( $row_terms as $row_term ) {
				$parent = null;
				$_terms = array_map( 'trim', explode( '>', $row_term ) );
				$total  = count( $_terms );

				foreach ( $_terms as $index => $_term ) {
					// Don't allow users without capabilities to create new categories.
					if ( ! current_user_can( 'manage_options' ) ) {
						break;
					}


					$term = wp_insert_term( $_term, $taxonomy, array( 'parent' => intval( $parent ) ) );

					if ( is_wp_error( $term ) ) {
						if ( $term->get_error_code() === 'term_exists' ) {
							// When term exists, error data should contain existing term id.
							$term_id = $term->get_error_data();
						} else {
							break; // We cannot continue on any other error.
						}
					} else {
						// New term.
						$term_id = $term['term_id'];
					}

					// Only requires assign the last category.
					if ( ( 1 + $index ) === $total ) {
						$categories[] = $term_id;
					} else {
						// Store parent to be able to insert or query categories based in parent ID.
						$parent = $term_id;
					}
					wp_set_post_terms( $post_id, $categories, $taxonomy, true );
				}
			}

			return $categories;
		}

		/**
		 * Implode CSV cell values using commas by default, and wrapping values
		 * which contain the separator.
		 *
		 * @since  3.7.5
		 * @param  array $values Values to implode.
		 * @return string
		 */
		public static function multi_question_category_implode_values( $values ) {
			$values_to_implode = array();

			foreach ( $values as $value ) {
				$value               = (string) is_scalar( $value ) ? $value : '';
				$values_to_implode[] = str_replace( ',', '\\,', $value );
			}

			return implode( ', ', $values_to_implode );
		}

		/**
		 * Explode CSV cell values using commas by default, and handling escaped
		 * separators.
		 *
		 * @since  3.7.5
		 * @param  string $value     Value to explode.
		 * @param  string $separator Separator separating each value. Defaults to comma.
		 * @return array
		 */
		public static function multi_question_category_explode_values( $value, $separator = ',' ) {
			$value  = str_replace( '\\,', '::separator::', $value );
			$values = explode( $separator, $value );
			$values = array_map( array( 'WN_LD_Quiz_Import_Export_Helper', 'multi_question_category_explode_values_formatter' ), $values );

			return $values;
		}

		/**
		 * Remove formatting and trim each value.
		 *
		 * @since  3.7.5
		 * @param  string $value Value to format.
		 * @return string
		 */
		public static function multi_question_category_explode_values_formatter( $value ) {
			return trim( str_replace( '::separator::', ',', $value ) );
		}



		/**
		 * Retrieves a page given its title.
		 *
		 * If more than one post uses the same title, the post with the smallest ID will be returned.
		 * Be careful: in case of more than one post having the same title, it will check the oldest
		 * publication date, not the smallest ID.
		 *
		 * Because this function uses the MySQL '=' comparison, $page_title will usually be matched
		 * as case-insensitive with default collation.
		 *
		 * @since 2.1.0
		 * @since 3.0.0 The `$post_type` parameter was added.
		 *
		 * @global wpdb $wpdb WordPress database abstraction object.
		 *
		 * @param string       $page_title Page title.
		 * @param string       $output     Optional. The required return type. One of OBJECT, ARRAY_A, or ARRAY_N, which
		 *                                 correspond to a WP_Post object, an associative array, or a numeric array,
		 *                                 respectively. Default OBJECT.
		 * @param string|array $post_type  Optional. Post type or array of post types. Default 'page'.
		 * @return WP_Post|array|null WP_Post (or array) on success, or null on failure.
		 */
		public static function wn_ldqie_get_page_by_title( $page_title, $output = OBJECT, $post_type = 'page' ) {
			global $wpdb;

			if ( is_array( $post_type ) ) {
				$post_type           = esc_sql( $post_type );
				$post_type_in_string = "'" . implode( "','", $post_type ) . "'";
				$sql                 = $wpdb->prepare(
					"SELECT ID
					FROM $wpdb->posts
					WHERE post_title = %s
					AND post_type IN ($post_type_in_string)",
					$page_title
				);
			} else {
				$sql = $wpdb->prepare(
					"SELECT ID
					FROM $wpdb->posts
					WHERE post_title = %s
					AND post_type = %s",
					$page_title,
					$post_type
				);
			}

			$page = $wpdb->get_var( $sql );

			if ( $page ) {
				return get_post( $page, $output );
			}

			return null;
		}



	}
}