<?php

/**
 * The file that defines the core plugin class
 *
 * A class definition that includes attributes and functions used across both the
 * public-facing side of the site and the admin area.
 *
 * @link       https://wooninjas.com/
 * @since      1.0.0
 *
 * @package    WN_LD_Quiz_Import_Export
 * @subpackage WN_LD_Quiz_Import_Export/includes
 */

/**
 * The core plugin class.
 *
 * This is used to define internationalization, admin-specific hooks, and
 * public-facing site hooks.
 *
 * Also maintains the unique identifier of this plugin as well as the current
 * version of the plugin.
 *
 * @since      1.0.0
 * @package    WN_LD_Quiz_Import_Export
 * @subpackage WN_LD_Quiz_Import_Export/includes
 * @author     WooNinjas <info@wooninjas.com>
 */
class WN_LD_Quiz_Import_Export {

	/**
	 * The loader that's responsible for maintaining and registering all hooks that power
	 * the plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      WN_LD_Quiz_Import_Export_Loader    $loader    Maintains and registers all hooks for the plugin.
	 */
	protected $loader;

	/**
	 * The unique identifier of this plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string    $plugin_name    The string used to uniquely identify this plugin.
	 */
	protected $plugin_name;

	/**
	 * The current version of the plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string    $version    The current version of the plugin.
	 */
	protected $version;

	/**
	 * Define the core functionality of the plugin.
	 *
	 * Set the plugin name and the plugin version that can be used throughout the plugin.
	 * Load the dependencies, define the locale, and set the hooks for the admin area and
	 * the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function __construct() {
		if ( defined( 'WN_LD_QUIZ_IMPORT_EXPORT_VERSION' ) ) {
			$this->version = WN_LD_QUIZ_IMPORT_EXPORT_VERSION;
		} else {
			$this->version = '1.0.0';
		}
		$this->plugin_name = 'wn-ld-quiz-import-export';

		$this->load_dependencies();
		$this->set_locale();
		$this->define_admin_hooks();
		$this->define_importer_hooks();
		$this->define_exporter_hooks();
	}

	/**
	 * Load the required dependencies for this plugin.
	 *
	 * Include the following files that make up the plugin:
	 *
	 * - WN_LD_Quiz_Import_Export_Loader. Orchestrates the hooks of the plugin.
	 * - WN_LD_Quiz_Import_Export_i18n. Defines internationalization functionality.
	 * - WN_LD_Quiz_Import_Export_Admin. Defines all hooks for the admin area.
	 * - WN_LD_Quiz_Import_Export_Public. Defines all hooks for the public side of the site.
	 *
	 * Create an instance of the loader which will be used to register the hooks
	 * with WordPress.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function load_dependencies() {

		/**
		 * The class responsible for orchestrating the actions and filters of the
		 * core plugin.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-wn-ld-quiz-import-export-loader.php';

		/**
		 * The class responsible for defining internationalization functionality
		 * of the plugin.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-wn-ld-quiz-import-export-i18n.php';

		/**
		 * The class responsible for defining global/helper functions
		 * of the plugin.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-wn-ld-quiz-import-export-helper.php';

		/**
		 * This class is responsible to define all cron related functionality
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-wn-ld-quiz-import-export-cron.php';

		/**
		 * The class responsible for defining all actions that occur in the admin area.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/class-wn-ld-quiz-import-export-admin.php';

		/**
		 * The class responsible for import functionality in admin area.
		 */
		if (!class_exists('PhpOffice\PhpSpreadsheet\Spreadsheet')) {
			if (version_compare(PHP_VERSION, '8.1.0', '>=')) {
				// If PHP version is 8.1.0 or above, require PHPSpreadsheet8
				require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/library/vendor/autoload.php';
			} else {
				// If PHP version is below 8.1.0, require PHPSpreadsheet7
				require_once plugin_dir_path(dirname(__FILE__)) . 'includes/library/phpspreadsheet7/vendor/autoload.php';
			}
		}
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/class-wn-ld-quiz-import-export-logging.php';
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/class-wn-ld-quiz-import-export-importer.php';
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/class-wn-ld-quiz-import-export-exporter.php';
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/class-wn-ld-quiz-import-export-json-exporter.php';
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/class-wn-ld-quiz-import-export-json-importer.php';

		/**
		 * The class responsible for defining all actions that occur to verify license.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/license/class-wn-ld-quiz-import-export-license.php';

		/**
		 * The class responsible for defining all actions that occur to verify and update plugin.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/license/class-wn-ld-quiz-import-export-plugin-updater.php';

		$this->loader = new WN_LD_Quiz_Import_Export_Loader();
	}

	/**
	 * Define the locale for this plugin for internationalization.
	 *
	 * Uses the WN_LD_Quiz_Import_Export_i18n class in order to set the domain and to register the hook
	 * with WordPress.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function set_locale() {
		$plugin_i18n = new WN_LD_Quiz_Import_Export_i18n();

		$this->loader->add_action( 'plugins_loaded', $plugin_i18n, 'load_plugin_textdomain' );
	}

	/**
	 * Register all of the hooks related to the admin area functionality of the plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function define_admin_hooks() {
		$plugin_license = new WN_LD_Quiz_Import_Export_License($this->get_plugin_name(), $this->get_version());
		$plugin_admin = new WN_LD_Quiz_Import_Export_Admin( $this->get_plugin_name(), $this->get_version(), $plugin_license );

		$this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'enqueue_styles' );
		$this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'enqueue_scripts' );
		$this->loader->add_action( 'admin_print_scripts', $plugin_admin, 'print_styles' );
		$this->loader->add_action( 'admin_print_styles', $plugin_admin, 'print_scripts' );

		//Plugin related menu, links and notifications
		$this->loader->add_action( 'admin_menu', $plugin_admin, 'admin_menu' );
		$this->loader->add_filter( 'user_has_cap', $plugin_admin, 'ld_add_extra_caps', 10, 3 );
		$this->loader->add_action( 'plugin_action_links_' . plugin_basename(WN_LD_QUIZ_IMPORT_EXPORT_FILE), $plugin_admin, 'plugin_action_links' );
		$this->loader->add_action( 'plugin_row_meta', $plugin_admin, 'plugin_row_meta', 10, 2 );
		$this->loader->add_action( 'admin_footer_text', $plugin_admin, 'admin_footer_text', 99 );
		$this->loader->add_action( 'admin_notices', $plugin_admin, 'plugin_review_notice' );
		$this->loader->add_action( 'admin_init', $plugin_admin, 'save_quiz_settings' );
		$this->loader->add_action( 'admin_init', $plugin_admin, 'save_settings' );
		$this->loader->add_action( 'admin_init', $plugin_admin, 'upgrade' );
		$this->loader->add_action( 'admin_notices', $plugin_admin, 'notice_settings_saved' );

		// JSON Importer AJAX
		$this->loader->add_action( 'wp_ajax_ld_qie_upload_json_file', $plugin_admin, 'ld_qie_upload_json_file_cb' );

		// Plugin license initialize and notices
		$this->loader->add_action( 'admin_init', $plugin_license, 'plugin_updater' );
		$this->loader->add_action( 'admin_init', $plugin_license, 'register_license_option' );
		$this->loader->add_action( 'admin_init', $plugin_license, 'activate_license' );
		$this->loader->add_action( 'admin_init', $plugin_license, 'deactivate_license' );
		$this->loader->add_action( 'admin_notices', $plugin_license, 'admin_notices' );
		$this->loader->add_action( 'admin_notices', $plugin_license, 'license_activation_notices' );

		if ( WN_LD_Quiz_Import_Export_Helper::doing_cron() ) {
			$this->loader->add_action( 'wn_ld_qie_weekly_scheduled_events', $plugin_license, 'weekly_license_check' );
		}

		$wn_ld_qie_settings = WN_LD_Quiz_Import_Export_Helper::get_settings();

		if( $wn_ld_qie_settings['debug_log'] === true ) { 
			// Check if WP_DEBUG_LOG is not already defined before defining it
			if (!defined('WP_DEBUG_LOG')) {
			    define('WP_DEBUG_LOG', true);
			}
		    ( new WN_LD_Quiz_Import_Export_Logging() )->copy_debug_log_to_new_file();
		}
	}

	/**
	 * Register all of the hooks related to the admin area import functionality of the plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	public function define_importer_hooks() {
		$wn_ld_qie_importer = new WN_LD_Quiz_Import_Export_Importer();

		$this->loader->add_action( 'wp_ajax_ld_qie_search_questions', $wn_ld_qie_importer, 'search_questions_cb' );
		$this->loader->add_action( 'wp_ajax_ld_qie_search_ld_post_type', $wn_ld_qie_importer, 'search_ld_post_type_cb' );
		$this->loader->add_action( 'wp_ajax_ld_qie_search_quizzes', $wn_ld_qie_importer, 'search_quizzes_cb' );
		$this->loader->add_action( 'wp_ajax_ld_qie_get_question_details', $wn_ld_qie_importer, 'get_question_details_cb' );
		$this->loader->add_action( 'wp_ajax_ld_qie_submit_quizzes_data', $wn_ld_qie_importer, 'import_quiz_cb' );
		$this->loader->add_action( 'wp_ajax_ld_qie_get_quizzes_list', $wn_ld_qie_importer, 'get_quizzes_list_cb' );
		$this->loader->add_action( 'wp_ajax_ld_qie_process_quiz_import', $wn_ld_qie_importer, 'process_import' );

		//TODO: Should be added in documentation
		if( defined('LD_QIE_UNIQUE_QUESTION_SLUG') && LD_QIE_UNIQUE_QUESTION_SLUG === true ) {
			//Set a unique slug value to short-circuit the slug iteration loop. Needed when user try to import thousands of questions with same title.
			//https://wordpress.stackexchange.com/questions/102349/faster-way-to-wp-insert-post-add-post-meta-in-bulk/102361
			$this->loader->add_filter( 'pre_wp_unique_post_slug', $wn_ld_qie_importer, 'pre_wp_unique_post_slug_cb', 10, 6);
		}

		$wn_ld_qie_settings = WN_LD_Quiz_Import_Export_Helper::get_settings();

		if( $wn_ld_qie_settings['activity_log'] === true ) {
			$this->loader->add_action( 'admin_init', $wn_ld_qie_importer, 'clear_debug_log' );
		}


		$this->loader->add_action( 'save_post_sfwd-quiz', $wn_ld_qie_importer, 'save_quiz_post', 15, 3 );
	}

	/**
	 * Register all of the hooks related to the admin area export functionality of the plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	public function define_exporter_hooks() {
		$wn_ld_qie_exporter = new WN_LD_Quiz_Import_Export_Exporter();

		$this->loader->add_filter( 'manage_sfwd-quiz_posts_columns', $wn_ld_qie_exporter, 'quiz_post_columns' );
		$this->loader->add_action( 'manage_sfwd-quiz_posts_custom_column', $wn_ld_qie_exporter, 'quiz_post_columns_data', 10, 2 );

		$this->loader->add_action( 'admin_init', $wn_ld_qie_exporter, 'ld_qie_export_cb' );
		$this->loader->add_filter( 'bulk_actions-edit-sfwd-quiz', $wn_ld_qie_exporter, 'ldmqie_register_export_bulk_actions' );
		$this->loader->add_filter( 'handle_bulk_actions-edit-sfwd-quiz', $wn_ld_qie_exporter, 'ldmqie_export_bulk_action_handler', 10, 3 );
	}

	/**
	 * Run the loader to execute all of the hooks with WordPress.
	 *
	 * @since    1.0.0
	 */
	public function run() {
		$this->loader->run();
	}

	/**
	 * The name of the plugin used to uniquely identify it within the context of
	 * WordPress and to define internationalization functionality.
	 *
	 * @since     1.0.0
	 * @return    string    The name of the plugin.
	 */
	public function get_plugin_name() {
		return $this->plugin_name;
	}

	/**
	 * The reference to the class that orchestrates the hooks with the plugin.
	 *
	 * @since     1.0.0
	 * @return    WN_LD_Quiz_Import_Export_Loader    Orchestrates the hooks of the plugin.
	 */
	public function get_loader() {
		return $this->loader;
	}

	/**
	 * Retrieve the version number of the plugin.
	 *
	 * @since     1.0.0
	 * @return    string    The version number of the plugin.
	 */
	public function get_version() {
		return $this->version;
	}

}
